@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-#include "config.h"
+#include <config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -23,17 +23,15 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #ifdef HAVE_STDDEF_H
 #include <stddef.h>
 #endif
 #include <locale.h>
 
-#ifdef GNOME
-#  include <gnome.h>
-#else
-#  include <gtk/gtk.h>
-#endif
+#include <gtk/gtk.h>
 
 #include "intl.h"
 #include "widgets.h"
@@ -41,6 +39,13 @@
 #include "message.h"
 #include "preferences.h"
 #include "dia_dirs.h"
+#include "diagramdata.h"
+#include "paper.h"
+#include "interface.h"
+
+#ifdef G_OS_WIN32
+#include <io.h> /* open, close */
+#endif
 
 struct DiaPreferences prefs;
 
@@ -51,10 +56,12 @@ enum DiaPrefType {
   PREF_UINT,
   PREF_REAL,
   PREF_UREAL,
-  PREF_COLOUR
+  PREF_COLOUR,
+  PREF_CHOICE,
+  PREF_STRING
 };
 
-struct DiaPrefsData {
+typedef struct _DiaPrefData {
   char *name;
   enum DiaPrefType type;
   int offset;
@@ -62,7 +69,9 @@ struct DiaPrefsData {
   int tab;
   char *label_text;
   GtkWidget *widget;
-};
+  gboolean hidden;
+  GList *(*choice_list_function)();
+} DiaPrefData;
 
 static int default_true = 1;
 static int default_false = 0;
@@ -71,8 +80,14 @@ static real default_real_zoom = 100.0;
 static int default_int_w = 500;
 static int default_int_h = 400;
 static int default_undo_depth = 15;
-static Color default_colour = { 0.5, 0.5, 0.5 };
-static Color pbreak_colour = { 0.0, 0.0, 0.6 };
+static guint default_recent_documents = 5;
+static Color default_colour = { 0.85, .90, .90 }; /* Grid colour */
+static Color pbreak_colour = { 0.0, 0.0, 0.6 }; 
+static guint default_dtree_width = 220;
+static guint default_dtree_height = 100;
+static guint default_dtree_dia_sort = DIA_TREE_SORT_INSERT;
+static guint default_dtree_obj_sort = DIA_TREE_SORT_INSERT;
+static const gchar *default_paper_name = NULL;
 
 struct DiaPrefsTab {
   char *title;
@@ -83,8 +98,10 @@ struct DiaPrefsTab {
 struct DiaPrefsTab prefs_tabs[] =
 {
   {N_("User Interface"), NULL, 0},
+  {N_("Diagram Defaults"), NULL, 0},
   {N_("View Defaults"), NULL, 0},
   {N_("Grid Lines"), NULL, 0},
+  {N_("Diagram Tree"), NULL, 0},
 };
 
 #define NUM_PREFS_TABS (sizeof(prefs_tabs)/sizeof(struct DiaPrefsTab))
@@ -96,12 +113,23 @@ struct DiaPrefsTab prefs_tabs[] =
 #define PREF_OFFSET(field)        ((int) ((char*) &((struct DiaPreferences *) 0)->field))
 #endif /* !offsetof */
 
-struct DiaPrefsData prefs_data[] =
+DiaPrefData prefs_data[] =
 {
   { "reset_tools_after_create", PREF_BOOLEAN, PREF_OFFSET(reset_tools_after_create), &default_true, 0, N_("Reset tools after create:") },
   { "compress_save", PREF_BOOLEAN, PREF_OFFSET(compress_save), &default_true, 0, N_("Compress saved files:") },
   { "undo_depth", PREF_UINT, PREF_OFFSET(undo_depth), &default_undo_depth, 0, N_("Number of undo levels:") },
   { "reverse_rubberbanding_intersects", PREF_BOOLEAN, PREF_OFFSET(reverse_rubberbanding_intersects), &default_true, 0, N_("Reverse dragging selects\nintersecting objects:") },
+  { "recent_documents_list_size", PREF_UINT, PREF_OFFSET(recent_documents_list_size), &default_recent_documents, 0, N_("Recent documents list size:") },
+  { "use_menu_bar", PREF_BOOLEAN, PREF_OFFSET(new_view.use_menu_bar), &default_false, 0, N_("Use menu bar:") },
+  { "toolbox_on_top", PREF_BOOLEAN, PREF_OFFSET(toolbox_on_top),
+    &default_false, 0, N_("Keep tool box on top of diagram windows:") },
+  
+  { NULL, PREF_NONE, 0, NULL, 1, N_("New diagram:") },
+  { "is_portrait", PREF_BOOLEAN, PREF_OFFSET(new_diagram.is_portrait), &default_true, 1, N_("Portrait:") },
+  { "new_diagram_papertype", PREF_CHOICE, PREF_OFFSET(new_diagram.papertype),
+    &default_paper_name, 1, N_("Paper type:"), NULL, FALSE,
+    get_paper_name_list },
+
   { NULL, PREF_NONE, 0, NULL, 1, N_("New window:") },
   { "new_view_width", PREF_UINT, PREF_OFFSET(new_view.width), &default_int_w, 1, N_("Width:") },
   { "new_view_height", PREF_UINT, PREF_OFFSET(new_view.height), &default_int_h, 1, N_("Height:") },
@@ -110,21 +138,48 @@ struct DiaPrefsData prefs_data[] =
   { NULL, PREF_NONE, 0, NULL, 1, N_("Connection Points:") },
   { "show_cx_pts", PREF_BOOLEAN, PREF_OFFSET(show_cx_pts), &default_true, 1, N_("Visible:") },
 
-  { NULL, PREF_NONE, 0, NULL, 2, N_("Grid:") },
-  { "grid_visible", PREF_BOOLEAN, PREF_OFFSET(grid.visible), &default_true, 2, N_("Visible:") },
-  { "grid_snap", PREF_BOOLEAN, PREF_OFFSET(grid.snap), &default_false, 2, N_("Snap to:") },
-  { "grid_x", PREF_UREAL, PREF_OFFSET(grid.x), &default_real_one, 2, N_("X Size:") },
-  { "grid_y", PREF_UREAL, PREF_OFFSET(grid.y), &default_real_one, 2, N_("Y Size:") },
-  { "grid_colour", PREF_COLOUR, PREF_OFFSET(grid.colour), &default_colour, 2, N_("Colour:") },
-  { "grid_solid", PREF_BOOLEAN, PREF_OFFSET(grid.solid), &default_true, 2, N_("Solid lines:") },
-
   { NULL, PREF_NONE, 0, NULL, 2, N_("Page breaks:") },
   { "pagebreak_visible", PREF_BOOLEAN, PREF_OFFSET(pagebreak.visible), &default_true, 2, N_("Visible:") },
   { "pagebreak_colour", PREF_COLOUR, PREF_OFFSET(pagebreak.colour), &pbreak_colour, 2, N_("Colour:") },
   { "pagebreak_solid", PREF_BOOLEAN, PREF_OFFSET(pagebreak.solid), &default_true, 2, N_("Solid lines:") },
+  
+  /*{ NULL, PREF_NONE, 0, NULL, 3, N_("Grid:") }, */
+  { "grid_visible", PREF_BOOLEAN, PREF_OFFSET(grid.visible), &default_true, 3, N_("Visible:") },
+  { "grid_snap", PREF_BOOLEAN, PREF_OFFSET(grid.snap), &default_false, 3, N_("Snap to:") },
+  { "grid_x", PREF_UREAL, PREF_OFFSET(grid.x), &default_real_one, 3, N_("X Size:") },
+  { "grid_y", PREF_UREAL, PREF_OFFSET(grid.y), &default_real_one, 3, N_("Y Size:") },
+  { "grid_colour", PREF_COLOUR, PREF_OFFSET(grid.colour), &default_colour, 3, N_("Colour:") },
+  { "grid_solid", PREF_BOOLEAN, PREF_OFFSET(grid.solid), &default_true, 3, N_("Solid lines:") },  
+
+  { "render_bounding_boxes", PREF_BOOLEAN,PREF_OFFSET(render_bounding_boxes),
+    &default_false,0,"render bounding boxes:",NULL, TRUE},
+
+  { "pretty_formated_xml", PREF_BOOLEAN,PREF_OFFSET(pretty_formated_xml),
+    &default_false,0,"pretty formated xml:",NULL, TRUE},
+
+  { "prefer_psprint", PREF_BOOLEAN,PREF_OFFSET(prefer_psprint),
+    &default_false,0,"prefer psprint:", NULL, TRUE},
+
+  { NULL, PREF_NONE, 0, NULL, 4, N_("Diagram tree window:") },
+  { "show_diagram_tree", PREF_BOOLEAN, PREF_OFFSET(dia_tree.show_tree),
+    &default_false, 4, N_("Show at startup:")},
+  { "diagram_tree_width", PREF_UINT, PREF_OFFSET(dia_tree.width),
+    &default_dtree_width, 4, N_("Default width:")},
+  { "diagram_tree_height", PREF_UINT, PREF_OFFSET(dia_tree.height),
+    &default_dtree_height, 4, N_("Default height:")},
+  { "diagram_tree_save_size", PREF_BOOLEAN, PREF_OFFSET(dia_tree.save_size),
+    &default_false, 4, N_("Remember last size:")},
+  { "diagram_tree_save_hidden", PREF_BOOLEAN, PREF_OFFSET(dia_tree.save_hidden),
+    &default_false, 4, N_("Save hidden object types:")},
+  { "diagram_tree_dia_sort", PREF_UINT, PREF_OFFSET(dia_tree.dia_sort),
+    &default_dtree_dia_sort, 4, "default diagram sort order", NULL, TRUE},
+  { "diagram_tree_obj_sort", PREF_UINT, PREF_OFFSET(dia_tree.obj_sort),
+    &default_dtree_obj_sort, 4, "default object sort order", NULL, TRUE},
+  { "diagram_tree_hidden", PREF_STRING, PREF_OFFSET(dia_tree.hidden),
+    &DIA_TREE_DEFAULT_HIDDEN, 4, "hidden type list", NULL, TRUE},
 };
 
-#define NUM_PREFS_DATA (sizeof(prefs_data)/sizeof(struct DiaPrefsData))
+#define NUM_PREFS_DATA (sizeof(prefs_data)/sizeof(DiaPrefData))
 
 static const GScannerConfig     dia_prefs_scanner_config =
 {
@@ -172,8 +227,8 @@ typedef enum {
 } DiaPrefsTokenType;
 
 static void prefs_create_dialog(void);
-static void prefs_set_value_in_widget(GtkWidget * widget, enum DiaPrefType type,  char *ptr);
-static void prefs_get_value_from_widget(GtkWidget * widget, enum DiaPrefType type, char *ptr);
+static void prefs_set_value_in_widget(GtkWidget * widget, DiaPrefData *data,  char *ptr);
+static void prefs_get_value_from_widget(GtkWidget * widget, DiaPrefData *data, char *ptr);
 static void prefs_update_dialog_from_prefs(void);
 static void prefs_update_prefs_from_dialog(void);
 static gint prefs_apply(GtkWidget *widget, gpointer data);
@@ -196,6 +251,12 @@ prefs_set_defaults(void)
   int i;
   char *ptr;
 
+  /* Since we can't call this in static initialization, we have to
+   * do it here.
+   */
+  if (default_paper_name == NULL)
+    default_paper_name = get_paper_name(get_default_paper());
+
   for (i=0;i<NUM_PREFS_DATA;i++) {
     ptr = (char *)&prefs + prefs_data[i].offset;
 
@@ -212,6 +273,10 @@ prefs_set_defaults(void)
     case PREF_COLOUR:
       *(Color *)ptr = *(Color *)prefs_data[i].default_value;
       break;
+    case PREF_CHOICE:
+    case PREF_STRING:
+      *(gchar **)ptr = *(gchar **)prefs_data[i].default_value;
+      break;
     case PREF_NONE:
       break;
     }
@@ -265,6 +330,11 @@ prefs_save(void)
     case PREF_COLOUR:
       fprintf(file, "%f %f %f\n", (double) ((Color *)ptr)->red,
 	      (double) ((Color *)ptr)->green, (double) ((Color *)ptr)->blue);
+      break;
+    case PREF_CHOICE:
+    case PREF_STRING:
+      fprintf(file, "\"%s\"\n", *(gchar **)ptr);
+      break;
     case PREF_NONE:
       break;
     }
@@ -336,6 +406,13 @@ prefs_parse_line(GScanner *scanner)
       return G_TOKEN_FLOAT;
     ((Color *)ptr)->blue = scanner->value.v_float;
 
+    break;
+  case PREF_CHOICE:
+  case PREF_STRING:
+    if (token != G_TOKEN_STRING)
+      return G_TOKEN_STRING;
+
+    *(char **)ptr = g_strdup(scanner->value.v_string);
     break;
   case PREF_NONE:
     break;
@@ -359,7 +436,7 @@ prefs_load(void)
   fd = open(filename, O_RDONLY);
 
   if (fd < 0) {
-    char *homedir = g_get_home_dir();
+    const gchar *homedir = g_get_home_dir();
 
     g_free(filename);
     filename = g_strconcat(homedir, G_DIR_SEPARATOR_S ".diarc", NULL);
@@ -378,15 +455,11 @@ prefs_load(void)
   g_scanner_input_file (scanner, fd);
 
   scanner->input_name = filename;
-
-  g_scanner_freeze_symbol_table(scanner);
   for (i = 0; i < NUM_PREFS_DATA; i++)
     if (prefs_data[i].type != PREF_NONE) {
       g_scanner_add_symbol(scanner, prefs_data[i].name,
 			   GINT_TO_POINTER(i));
     }
-  g_scanner_thaw_symbol_table(scanner);
-  
   while (1) {
     if (g_scanner_peek_next_token(scanner) == G_TOKEN_EOF) {
       break;
@@ -411,32 +484,17 @@ prefs_load(void)
   }
   
   g_scanner_destroy (scanner);
-  
+ 
   close(fd);
-}
-
-static gint
-prefs_okay(GtkWidget *widget, gpointer data)
-{
-  gint ret = prefs_apply(widget,data);
-  gtk_widget_hide(widget);
-  return ret;
-}
-
-static gint
-prefs_apply(GtkWidget *widget, gpointer data)
-{
-  prefs_update_prefs_from_dialog();
-  prefs_save();
-  diagram_redraw_all();
-  return 1;
+  render_bounding_boxes = prefs.render_bounding_boxes;
+  pretty_formated_xml = prefs.pretty_formated_xml;
 }
 
 static void
-prefs_set_value_in_widget(GtkWidget * widget, enum DiaPrefType type,
+prefs_set_value_in_widget(GtkWidget * widget, DiaPrefData *data,
 			  char *ptr)
 {
-  switch(type) {
+  switch(data->type) {
   case PREF_BOOLEAN:
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), *((int *)ptr));
     break;
@@ -453,16 +511,32 @@ prefs_set_value_in_widget(GtkWidget * widget, enum DiaPrefType type,
   case PREF_COLOUR:
     dia_color_selector_set_color(DIACOLORSELECTOR(widget), (Color *)ptr);
     break;
+  case PREF_CHOICE: {
+    GList *names = (data->choice_list_function)();
+    int index;
+    for (index = 0; names != NULL; names = g_list_next(names), index++) {
+      if (!strcmp(*((gchar**)ptr), (gchar *)names->data))
+	break;
+    }
+    if (names == NULL) return;
+    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), index);
+    gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(widget))), index);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(widget))))), TRUE);
+    break;
+  }
+  case PREF_STRING:
+    gtk_entry_set_text(GTK_ENTRY(widget), (gchar *)(*((gchar **)ptr)));
+    break;
   case PREF_NONE:
     break;
   }
 }
 
 static void
-prefs_get_value_from_widget(GtkWidget * widget, enum DiaPrefType type,
+prefs_get_value_from_widget(GtkWidget * widget, DiaPrefData *data,
 			    char *ptr)
 {
-  switch(type) {
+  switch(data->type) {
   case PREF_BOOLEAN:
     *((int *)ptr) = GTK_TOGGLE_BUTTON(widget)->active;    
     break;
@@ -479,6 +553,16 @@ prefs_get_value_from_widget(GtkWidget * widget, enum DiaPrefType type,
   case PREF_COLOUR:
     dia_color_selector_get_color(DIACOLORSELECTOR(widget), (Color *)ptr);
     break;
+  case PREF_CHOICE: {
+    int index = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
+    GList *names = (data->choice_list_function)();
+    *((gchar **)ptr) = g_strdup((gchar *)g_list_nth_data(names, index));
+    break;
+  }
+  case PREF_STRING:
+    *((gchar **)ptr) = (gchar *)
+      gtk_entry_get_text(GTK_ENTRY(widget));
+    break;
   case PREF_NONE:
     break;
   }
@@ -488,17 +572,16 @@ static void
 prefs_boolean_toggle(GtkWidget *widget, gpointer data)
 {
   guint active = GTK_TOGGLE_BUTTON(widget)->active;
-  GtkWidget *label = GTK_BUTTON(widget)->child;
-  gtk_label_set(GTK_LABEL(label), active ? _("Yes") : _("No"));
+  gtk_button_set_label(GTK_BUTTON(widget), active ? _("Yes") : _("No"));
 }
 
 static GtkWidget *
-prefs_get_property_widget(enum DiaPrefType type)
+prefs_get_property_widget(DiaPrefData *data)
 {
   GtkWidget *widget = NULL;
   GtkAdjustment *adj;
   
-  switch(type) {
+  switch(data->type) {
   case PREF_BOOLEAN:
     widget = gtk_toggle_button_new_with_label (_("No"));
     gtk_signal_connect (GTK_OBJECT (widget), "toggled",
@@ -539,6 +622,26 @@ prefs_get_property_widget(enum DiaPrefType type)
   case PREF_COLOUR:
     widget = dia_color_selector_new();
     break;
+  case PREF_STRING:
+    widget = gtk_entry_new();
+    break;
+  case PREF_CHOICE: {
+    GtkWidget *menu;
+    GList *names;
+    GSList *group = NULL;
+    widget = gtk_option_menu_new();
+    menu = gtk_menu_new();
+    for (names = (data->choice_list_function)(); names != NULL;
+	 names = g_list_next(names)) {
+      GtkWidget *menuitem =
+	gtk_radio_menu_item_new_with_label(group, (gchar *)names->data);
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+      group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
+    }
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(widget), menu);
+    gtk_widget_show_all(menu);
+    break;
+  }
   case PREF_NONE:
     widget = NULL;
     break;
@@ -548,12 +651,29 @@ prefs_get_property_widget(enum DiaPrefType type)
   return widget;
 }
 
+static gint
+prefs_respond(GtkWidget *widget, 
+                   gint       response_id,
+                   gpointer   data)
+{
+  if (   response_id == GTK_RESPONSE_APPLY 
+      || response_id == GTK_RESPONSE_OK) {
+    prefs_update_prefs_from_dialog();
+    prefs_save();
+    diagram_redraw_all();
+  }
+
+  if (response_id != GTK_RESPONSE_APPLY)
+    gtk_widget_hide(widget);
+
+  return 0;
+}
+
 static void
 prefs_create_dialog(void)
 {
   GtkWidget *notebook_page;
   GtkWidget *label;
-  GtkWidget *button;
   GtkWidget *dialog_vbox;
   GtkWidget *notebook;
   GtkWidget *table;
@@ -562,70 +682,28 @@ prefs_create_dialog(void)
   if (prefs_dialog != NULL)
     return;
 
-#ifdef GNOME
-  prefs_dialog = gnome_dialog_new(_("Preferences"),
-				  GNOME_STOCK_BUTTON_OK,
-				  GNOME_STOCK_BUTTON_APPLY,
-				  GNOME_STOCK_BUTTON_CLOSE, NULL);
-  gnome_dialog_set_default(GNOME_DIALOG(prefs_dialog), 1);
-
-  dialog_vbox = GNOME_DIALOG(prefs_dialog)->vbox;
-#else
-  prefs_dialog = gtk_dialog_new();
-  gtk_window_set_title (GTK_WINDOW (prefs_dialog), _("Preferences"));
-  gtk_container_set_border_width (GTK_CONTAINER (prefs_dialog), 2);
-  gtk_window_set_policy (GTK_WINDOW (prefs_dialog),
-			 FALSE, TRUE, FALSE);
+  prefs_dialog = gtk_dialog_new_with_buttons(
+			_("Preferences"),
+			GTK_WINDOW(interface_get_toolbox_shell()),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+			GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
+			GTK_STOCK_OK, GTK_RESPONSE_OK,
+			NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG(prefs_dialog), GTK_RESPONSE_OK);
+  gtk_window_set_resizable (GTK_WINDOW (prefs_dialog), TRUE);
 
   dialog_vbox = GTK_DIALOG (prefs_dialog)->vbox;
-#endif
   
-  gtk_window_set_wmclass (GTK_WINDOW (prefs_dialog),
-			  "preferences_window", "Dia");
+  gtk_window_set_role (GTK_WINDOW (prefs_dialog), "preferences_window");
 
-
-#ifdef GNOME
-  gnome_dialog_button_connect_object(GNOME_DIALOG(prefs_dialog), 0,
-				     GTK_SIGNAL_FUNC(prefs_okay),
-				     GTK_OBJECT(prefs_dialog));
-  gnome_dialog_button_connect_object(GNOME_DIALOG(prefs_dialog), 1,
-				     GTK_SIGNAL_FUNC(prefs_apply),
-				     GTK_OBJECT(prefs_dialog));
-  gnome_dialog_button_connect_object(GNOME_DIALOG(prefs_dialog), 2,
-				     GTK_SIGNAL_FUNC(gtk_widget_hide),
-				     GTK_OBJECT(prefs_dialog));
-#else
-  button = gtk_button_new_with_label( _("OK") );
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_dialog)->action_area), 
-		      button, TRUE, TRUE, 0);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC(prefs_okay),
-			     GTK_OBJECT(prefs_dialog));
-  gtk_widget_show (button);
-
-  button = gtk_button_new_with_label( _("Apply") );
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_dialog)->action_area), 
-		      button, TRUE, TRUE, 0);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC(prefs_apply),
-		      NULL);
-  gtk_widget_show (button);
-
-  button = gtk_button_new_with_label( _("Close") );
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_dialog)->action_area), 
-		      button, TRUE, TRUE, 0);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC(gtk_widget_hide),
-			     GTK_OBJECT(prefs_dialog));
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-#endif
+  g_signal_connect(G_OBJECT (prefs_dialog), "response",
+                   G_CALLBACK (prefs_respond), NULL);
 
   gtk_signal_connect (GTK_OBJECT (prefs_dialog), "delete_event",
 		      GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
+  gtk_signal_connect (GTK_OBJECT (prefs_dialog), "destroy",
+		      GTK_SIGNAL_FUNC(gtk_widget_destroyed), &prefs_dialog);
 
   notebook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
@@ -665,7 +743,9 @@ prefs_create_dialog(void)
     GtkTable *table;
     GtkWidget *widget;
     int row;
-    
+
+    if (prefs_data[i].hidden) continue;
+
     label = gtk_label_new (gettext(prefs_data[i].label_text));
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.3);
     gtk_widget_show (label);
@@ -676,12 +756,12 @@ prefs_create_dialog(void)
 		      row, row + 1,
 		      GTK_FILL, GTK_FILL, 1, 1);
 
-    widget = prefs_get_property_widget(prefs_data[i].type);
+    widget = prefs_get_property_widget(&prefs_data[i]);
     prefs_data[i].widget = widget;
     if (widget != NULL) {
       gtk_table_attach (table, widget, 1, 2,
 			row, row + 1,
-			GTK_EXPAND | GTK_FILL, GTK_FILL, 1, 1);
+			GTK_FILL, GTK_FILL, 1, 1);
     }
     
   }
@@ -697,10 +777,11 @@ prefs_update_prefs_from_dialog(void)
   char *ptr;
   
   for (i=0;i<NUM_PREFS_DATA;i++) {
+    if (prefs_data[i].hidden) continue;
     widget = prefs_data[i].widget;
     ptr = (char *)&prefs + prefs_data[i].offset;
     
-    prefs_get_value_from_widget(widget, prefs_data[i].type,  ptr);
+    prefs_get_value_from_widget(widget, &prefs_data[i],  ptr);
   }
 }
 
@@ -712,14 +793,10 @@ prefs_update_dialog_from_prefs(void)
   char *ptr;
   
   for (i=0;i<NUM_PREFS_DATA;i++) {
+    if (prefs_data[i].hidden) continue;
     widget = prefs_data[i].widget;
     ptr = (char *)&prefs + prefs_data[i].offset;
     
-    prefs_set_value_in_widget(widget, prefs_data[i].type,  ptr);
+    prefs_set_value_in_widget(widget, &prefs_data[i],  ptr);
   }
 }
-
-
-
-
-
