@@ -18,6 +18,7 @@
 #include <config.h>
 
 #include "attributes.h"
+#include "intl.h"
 
 static Color attributes_foreground = { 0.0f, 0.0f, 0.0f };
 static Color attributes_background = { 1.0f, 1.0f, 1.0f };
@@ -30,7 +31,7 @@ static Arrow attributes_end_arrow = { ARROW_NONE, 0.8, 0.8 };
 static LineStyle attributes_linestyle = LINESTYLE_SOLID;
 static real attributes_dash_length = 1.0;
 
-static Font *attributes_font = NULL;
+static DiaFont *attributes_font = NULL;
 static real attributes_font_height = 0.8;
 
 Color 
@@ -121,18 +122,22 @@ attributes_set_default_line_style(LineStyle style, real dash_length)
 }
 
 void
-attributes_get_default_font(Font **font, real *font_height)
+attributes_get_default_font(DiaFont **font, real *font_height)
 {
-  if (!attributes_font)
-    attributes_font = font_getfont("Courier");
-  if (font)
-    *font = attributes_font;
-  if (font_height)
-    *font_height = attributes_font_height;
+	if (!attributes_font) {
+		attributes_font = dia_font_new_from_style(DIA_FONT_SANS,
+                                              attributes_font_height);
+	}
+	if (font)
+		*font = dia_font_ref(attributes_font);
+	if (font_height)
+		*font_height = attributes_font_height;
 }
+
 void
-attributes_set_default_font(Font *font, real font_height)
+attributes_set_default_font(DiaFont *font, real font_height)
 {
-  attributes_font = font;
+  dia_font_unref(attributes_font);  
+  attributes_font = dia_font_ref(font);
   attributes_font_height = font_height;
 }
