@@ -1,4 +1,4 @@
-/* Dia -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program -*- c -*-
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,22 @@
 
 #include <glib.h>
 
-typedef struct _DiagramData DiagramData;
-typedef struct _Layer Layer;
-
-#include "object.h"
+#include "diatypes.h"
+/* #include "object.h" later after declaring types */
+#include "color.h"
+#include "geometry.h"
+#include "diavar.h"
 #include "paper.h"
 
+struct _NewDiagramData {
+  gchar *papertype;
+  gfloat tmargin, bmargin, lmargin, rmargin;
+  gboolean is_portrait;
+  gfloat scaling;
+  gboolean fitto;
+  gint fitwidth, fitheight;
+};
+
 struct _DiagramData {
   Rectangle extents;      /* The extents of the diagram        */
 
@@ -64,12 +74,21 @@ struct _Layer {
 			     in the same layer! */
 
   int visible;
+
+  DiagramData *parent_diagram; /* Back-pointer to the diagram.  This
+				  must only be set by functions internal
+				  to the diagram, and accessed via
+				  layer_get_parent_diagram() */
 };
 
-DiagramData *new_diagram_data(void);
+#include "object.h"
+
+DIAVAR int render_bounding_boxes;
+
+DiagramData *new_diagram_data(NewDiagramData *prefs);
 void diagram_data_destroy(DiagramData *data);
 
-Layer *new_layer(char *name);
+Layer *new_layer (char *name, DiagramData *parent);
 void layer_destroy(Layer *layer);
 
 void data_raise_layer(DiagramData *data, Layer *layer);
@@ -82,17 +101,17 @@ void data_delete_layer(DiagramData *data, Layer *layer);
 void data_select(DiagramData *data, Object *obj);
 void data_unselect(DiagramData *data, Object *obj);
 void data_remove_all_selected(DiagramData *data);
-int data_update_extents(DiagramData *data); /* returns true if changed. */
+gboolean data_update_extents(DiagramData *data); /* returns true if changed. */
 GList *data_get_sorted_selected(DiagramData *data);
 GList *data_get_sorted_selected_remove(DiagramData *data);
 
-typedef void (*ObjectRenderer)(Object *obj, Renderer *renderer,
+typedef void (*ObjectRenderer)(Object *obj, DiaRenderer *renderer,
 			       int active_layer,
 			       gpointer data);
-void data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
+void data_render(DiagramData *data, DiaRenderer *renderer, Rectangle *update,
 		 ObjectRenderer obj_renderer /* Can be NULL */,
 		 gpointer gdata);  
-void layer_render(Layer *layer, Renderer *renderer, Rectangle *update,
+void layer_render(Layer *layer, DiaRenderer *renderer, Rectangle *update,
 		  ObjectRenderer obj_renderer /* Can be NULL */,
 		  gpointer data,
 		  int active_layer);
@@ -109,13 +128,16 @@ GList *layer_find_objects_in_rectangle(Layer *layer, Rectangle *rect);
 Object *layer_find_closest_object(Layer *layer, Point *pos, real maxdist);
 real layer_find_closest_connectionpoint(Layer *layer,
 					ConnectionPoint **closest,
-					Point *pos);
+					Point *pos,
+					Object *notthis);
 int layer_update_extents(Layer *layer); /* returns true if changed. */
 void layer_replace_object_with_list(Layer *layer, Object *obj,
 				    GList *list);
 void layer_set_object_list(Layer *layer, GList *list);
+DiagramData *layer_get_parent_diagram(Layer *layer);
 /* Make sure all objects that are in the layer and not in the new
    list eventually gets destroyed. */
+
 #endif /* DIAGRAMDATA_H */
 
 
