@@ -1,6 +1,9 @@
 /* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
+ * Matrix code from GIMP:app/transform_tool.c
+ * Copyright (C) 1995 Spencer Kimball and Peter Mattis
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -17,6 +20,8 @@
  */
 
 /* include glib.h first, so we don't pick up its inline functions as well */
+#include <config.h>
+
 #include <glib.h>
 /* include normal versions of the inlined functions here ... */
 #undef G_INLINE_FUNC
@@ -25,7 +30,7 @@
 #include "geometry.h"
 
 void
-rectangle_union(Rectangle *r1, Rectangle *r2)
+rectangle_union(Rectangle *r1, const Rectangle *r2)
 {
   r1->top = MIN( r1->top, r2->top );
   r1->bottom = MAX( r1->bottom, r2->bottom );
@@ -34,7 +39,7 @@ rectangle_union(Rectangle *r1, Rectangle *r2)
 }
 
 void
-int_rectangle_union(IntRectangle *r1, IntRectangle *r2)
+int_rectangle_union(IntRectangle *r1, const IntRectangle *r2)
 {
   r1->top = MIN( r1->top, r2->top );
   r1->bottom = MAX( r1->bottom, r2->bottom );
@@ -43,7 +48,7 @@ int_rectangle_union(IntRectangle *r1, IntRectangle *r2)
 }
 
 void
-rectangle_intersection(Rectangle *r1, Rectangle *r2)
+rectangle_intersection(Rectangle *r1, const Rectangle *r2)
 {
   r1->top = MAX( r1->top, r2->top );
   r1->bottom = MIN( r1->bottom, r2->bottom );
@@ -58,7 +63,7 @@ rectangle_intersection(Rectangle *r1, Rectangle *r2)
 }
 
 int
-rectangle_intersects(Rectangle *r1, Rectangle *r2)
+rectangle_intersects(const Rectangle *r1, const Rectangle *r2)
 {
   if ( (r1->right < r2->left) ||
        (r1->left > r2->right) ||
@@ -70,7 +75,7 @@ rectangle_intersects(Rectangle *r1, Rectangle *r2)
 }
 
 int
-point_in_rectangle(Rectangle* r, Point *p)
+point_in_rectangle(const Rectangle* r, const Point *p)
 {
   if ( (p->x < r->left) ||
        (p->x > r->right) ||
@@ -82,7 +87,7 @@ point_in_rectangle(Rectangle* r, Point *p)
 }
 
 int
-rectangle_in_rectangle(Rectangle* outer, Rectangle *inner)
+rectangle_in_rectangle(const Rectangle* outer, const Rectangle *inner)
 {
   if ( (inner->left < outer->left) ||
        (inner->right > outer->right) ||
@@ -94,7 +99,7 @@ rectangle_in_rectangle(Rectangle* outer, Rectangle *inner)
 }
 
 void
-rectangle_add_point(Rectangle *r, Point *p)
+rectangle_add_point(Rectangle *r, const Point *p)
 {
   if (p->x < r->left)
     r->left = p->x;
@@ -115,7 +120,7 @@ rectangle_add_point(Rectangle *r, Point *p)
  * on the rectangle is returned.
  */
 real
-distance_rectangle_point(Rectangle *rect, Point *point)
+distance_rectangle_point(const Rectangle *rect, const Point *point)
 {
   real dx = 0.0;
   real dy = 0.0;
@@ -145,8 +150,8 @@ distance_rectangle_point(Rectangle *rect, Point *point)
  * end of line segment.
  */
 real
-distance_line_point(Point *line_start, Point *line_end,
-		    real line_width, Point *point)
+distance_line_point(const Point *line_start, const Point *line_end,
+		    real line_width, const Point *point)
 {
   Point v1, v2;
   real v1_lensq;
@@ -192,13 +197,14 @@ distance_line_point(Point *line_start, Point *line_end,
 
 /* returns 1 iff the line crosses the ray from (-Inf, rayend.y) to rayend */
 static guint
-line_crosses_ray(Point *line_start, Point *line_end, Point *rayend)
+line_crosses_ray(const Point *line_start, 
+                 const Point *line_end, const Point *rayend)
 {
   coord xpos;
 
   /* swap end points if necessary */
   if (line_start->y > line_end->y) {
-    Point *tmp;
+    const Point *tmp;
 
     tmp = line_start;
     line_start = line_end;
@@ -217,8 +223,8 @@ line_crosses_ray(Point *line_start, Point *line_end, Point *rayend)
 }
 
 real
-distance_polygon_point(Point *poly, guint npoints, real line_width,
-		       Point *point)
+distance_polygon_point(const Point *poly, guint npoints, real line_width,
+		       const Point *point)
 {
   guint i, last = npoints - 1;
   real line_dist = G_MAXFLOAT;
@@ -246,8 +252,11 @@ distance_polygon_point(Point *poly, guint npoints, real line_width,
 
 /* if cross is not NULL, it will be incremented for each ray crossing */
 static real
-bez_point_distance_and_ray_crosses(Point *b1, Point *b2, Point *b3, Point *b4,
-				   real line_width, Point *point, guint *cross)
+bez_point_distance_and_ray_crosses(const Point *b1, 
+                                   const Point *b2, const Point *b3, 
+                                   const Point *b4,
+				   real line_width, const Point *point, 
+                                   guint *cross)
 {
   static gboolean calculated_coeff = FALSE;
   static real coeff[NBEZ_SEGS+1][4];
@@ -291,16 +300,17 @@ bez_point_distance_and_ray_crosses(Point *b1, Point *b2, Point *b3, Point *b4,
 }
 
 real
-distance_bez_seg_point(Point *b1, Point *b2, Point *b3, Point *b4,
-		       real line_width, Point *point)
+distance_bez_seg_point(const Point *b1, const Point *b2, 
+                       const Point *b3, const Point *b4,
+		       real line_width, const Point *point)
 {
   return bez_point_distance_and_ray_crosses(b1, b2, b3, b4,
 					    line_width, point, NULL);
 }
 			     
 real
-distance_bez_line_point(BezPoint *b, guint npoints,
-			real line_width, Point *point)
+distance_bez_line_point(const BezPoint *b, guint npoints,
+			real line_width, const Point *point)
 {
   Point last;
   guint i;
@@ -336,8 +346,8 @@ distance_bez_line_point(BezPoint *b, guint npoints,
 }
 
 real
-distance_bez_shape_point(BezPoint *b, guint npoints,
-			real line_width, Point *point)
+distance_bez_shape_point(const BezPoint *b, guint npoints,
+                         real line_width, const Point *point)
 {
   Point last;
   guint i;
@@ -380,9 +390,13 @@ distance_bez_shape_point(BezPoint *b, guint npoints,
 }
 
 real
-distance_ellipse_point(Point *centre, real width, real height,
-		       real line_width, Point *point)
+distance_ellipse_point(const Point *centre, real width, real height,
+		       real line_width, const Point *point)
 {
+  /* A faster intersection method would be to scaled the ellipse and the
+   * point to where the ellipse is a circle, intersect with the circle, 
+   * then scale back.
+   */
   real w2 = width*width, h2 = height*height;
   real scale, rad, dist;
   Point pt;
@@ -408,3 +422,142 @@ distance_ellipse_point(Point *centre, real width, real height,
   else
     return dist - rad;
 }
+
+
+void
+transform_point (Matrix m, Point *src, Point *dest)
+{
+  real xx, yy, ww;
+
+  xx = m[0][0] * src->x + m[0][1] * src->y + m[0][2];
+  yy = m[1][0] * src->x + m[1][1] * src->y + m[1][2];
+  ww = m[2][0] * src->x + m[2][1] * src->y + m[2][2];
+
+  if (!ww)
+    ww = 1.0;
+
+  dest->x = xx / ww;
+  dest->y = yy / ww;
+}
+
+void
+mult_matrix (Matrix m1, Matrix m2)
+{
+  Matrix result;
+  int i, j, k;
+
+  for (i = 0; i < 3; i++)
+    for (j = 0; j < 3; j++)
+      {
+	result [i][j] = 0.0;
+	for (k = 0; k < 3; k++)
+	  result [i][j] += m1 [i][k] * m2[k][j];
+      }
+
+  /*  copy the result into matrix 2  */
+  for (i = 0; i < 3; i++)
+    for (j = 0; j < 3; j++)
+      m2 [i][j] = result [i][j];
+}
+
+void
+identity_matrix (Matrix m)
+{
+  int i, j;
+
+  for (i = 0; i < 3; i++)
+    for (j = 0; j < 3; j++)
+      m[i][j] = (i == j) ? 1 : 0;
+
+}
+
+void
+translate_matrix (Matrix m, real x, real y)
+{
+  Matrix trans;
+
+  identity_matrix (trans);
+  trans[0][2] = x;
+  trans[1][2] = y;
+  mult_matrix (trans, m);
+}
+
+void
+scale_matrix (Matrix m, real x, real y)
+{
+  Matrix scale;
+
+  identity_matrix (scale);
+  scale[0][0] = x;
+  scale[1][1] = y;
+  mult_matrix (scale, m);
+}
+
+void
+rotate_matrix (Matrix m, real theta)
+{
+  Matrix rotate;
+  real cos_theta, sin_theta;
+
+  cos_theta = cos (theta);
+  sin_theta = sin (theta);
+
+  identity_matrix (rotate);
+  rotate[0][0] = cos_theta;
+  rotate[0][1] = -sin_theta;
+  rotate[1][0] = sin_theta;
+  rotate[1][1] = cos_theta;
+  mult_matrix (rotate, m);
+}
+
+void
+xshear_matrix (Matrix m, real shear)
+{
+  Matrix shear_m;
+
+  identity_matrix (shear_m);
+  shear_m[0][1] = shear;
+  mult_matrix (shear_m, m);
+}
+
+void
+yshear_matrix (Matrix m, real shear)
+{
+  Matrix shear_m;
+
+  identity_matrix (shear_m);
+  shear_m[1][0] = shear;
+  mult_matrix (shear_m, m);
+}
+
+/*  find the determinate for a 3x3 matrix  */
+static real
+determinate (Matrix m)
+{
+  int i;
+  double det = 0;
+
+  for (i = 0; i < 3; i ++)
+    {
+      det += m[0][i] * m[1][(i+1)%3] * m[2][(i+2)%3];
+      det -= m[2][i] * m[1][(i+1)%3] * m[0][(i+2)%3];
+    }
+
+  return det;
+}
+
+
+
+void
+point_convex(Point *dst, const Point *src1, const Point *src2, real alpha)
+{
+  /* Make convex combination of src1 and src2:
+     dst = alpha * src1 + (1-alpha) * src2;
+  */
+  point_copy(dst,src1);
+  point_scale(dst,alpha);
+  point_add_scaled(dst,src2,1.0 - alpha);
+}
+
+
+
