@@ -2,9 +2,7 @@
  * Copyright (C) 1998 Alexander Larsson
  *
  * GRAFCET charts support for Dia 
- * Copyright (C) 2000 Cyrille Chepelov
- *
- * This file has been forked from objects/ER/entity.c
+ * Copyright (C) 2000, 2001 Cyrille Chepelov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +24,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,16 +33,16 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
-#include "lazyprops.h"
+#include "properties.h"
 
 #include "grafcet.h"
 
 #include "pixmaps/etape.xpm"
 
-#define STEP_FONT "Helvetica-Bold"
+#define STEP_FONT (DIA_FONT_SANS|DIA_FONT_BOLD)
 #define STEP_FONT_HEIGHT 1
 #define STEP_LINE_WIDTH GRAFCET_GENERAL_LINE_WIDTH 
 #define STEP_WIDTH 3.0
@@ -65,25 +62,7 @@ typedef enum {
   STEP_MACROCALL,
   STEP_SUBPCALL} StepType;
 
-typedef struct _Step Step;
-typedef struct _StepPropertiesDialog StepPropertiesDialog;
-typedef struct _StepDefaults StepDefaults;
-typedef struct _StepDefaultsDialog StepDefaultsDialog;
-typedef struct _StepState StepState;
-
-struct _StepState {
-  ObjectState obj_state;
-
-  char *id;
-  int active;
-  StepType type;
-
-  Font *font;
-  real font_size;
-  Color font_color;
-};  
-
-struct _Step {
+typedef struct _Step {
   Element element;
 
   ConnectionPoint connections[4];
@@ -92,7 +71,7 @@ struct _Step {
   int active;
   StepType type;
 
-  Font *font;
+  DiaFont *font;
   real font_size;
   Color font_color;
 
@@ -102,72 +81,39 @@ struct _Step {
   /* These are useful points for drawing. 
      Must be in sequence, A first, Z last. */
   Point A,B,C,D,E,F,G,H,I,J,Z;
-};
-
-struct _StepPropertiesDialog {
-  AttributeDialog dialog;
-  Step *parent;
-
-  StringAttribute id;
-  BoolAttribute active;
-  EnumAttribute type;
-
-  FontAttribute font;
-  FontHeightAttribute font_size;
-  ColorAttribute font_color;
-};
-
-struct _StepDefaults {
-  Font *font;
-  real font_size;
-  Color font_color;
-};
-
-struct _StepDefaultsDialog {
-  AttributeDialog dialog;
-  StepDefaults *parent;
-
-  FontAttribute font;
-  FontHeightAttribute font_size;
-  ColorAttribute font_color;
-};
+} Step;
 
 static real step_distance_from(Step *step, Point *point);
 static void step_select(Step *step, Point *clicked_point,
-			Renderer *interactive_renderer);
+			DiaRenderer *interactive_renderer);
 static void step_move_handle(Step *step, Handle *handle,
 			    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void step_move(Step *step, Point *to);
-static void step_draw(Step *step, Renderer *renderer);
+static void step_draw(Step *step, DiaRenderer *renderer);
 static void step_update_data(Step *step);
 static Object *step_create(Point *startpoint,
 			     void *user_data,
 			     Handle **handle1,
 			     Handle **handle2);
 static void step_destroy(Step *step);
-static Object *step_copy(Step *step);
-static PROPDLG_TYPE step_get_properties(Step *step);
-static ObjectChange *step_apply_properties(Step *step);
 
-static void step_save(Step *step, ObjectNode obj_node,
-			const char *filename);
+static void step_been_renamed(const gchar *sid);
+
 static Object *step_load(ObjectNode obj_node, int version,
 			   const char *filename);
-static StepState *step_get_state(Step *step);
-static void step_set_state(Step *step, StepState *state);
-static PROPDLG_TYPE step_get_defaults(void);
-static void step_apply_defaults(void);
-static void step_init_defaults(void);
-
-
+static PropDescription *step_describe_props(Step *step);
+static void step_get_props(Step *step, 
+                                 GPtrArray *props);
+static void step_set_props(Step *step, 
+                                 GPtrArray *props);
 
 static ObjectTypeOps step_type_ops =
 {
   (CreateFunc) step_create,
-  (LoadFunc)   step_load,
-  (SaveFunc)   step_save,
-  (GetDefaultsFunc)   step_get_defaults, 
-  (ApplyDefaultsFunc) step_apply_defaults
+  (LoadFunc)   step_load/*using properties*/,
+  (SaveFunc)   object_save_using_properties,
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL,
 };
 
 ObjectType step_type =
@@ -184,18 +130,82 @@ static ObjectOps step_ops = {
   (DrawFunc)            step_draw,
   (DistanceFunc)        step_distance_from,
   (SelectFunc)          step_select,
-  (CopyFunc)            step_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            step_move,
   (MoveHandleFunc)      step_move_handle,
-  (GetPropertiesFunc)   step_get_properties,
-  (ApplyPropertiesFunc) step_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   step_describe_props,
+  (GetPropsFunc)        step_get_props,
+  (SetPropsFunc)        step_set_props
 };
 
-static StepPropertiesDialog *step_properties_dialog = NULL;
-static StepDefaultsDialog *step_defaults_dialog = NULL;
-static StepDefaults defaults;
+PropEnumData step_style[] = {
+  { N_("Regular step"),STEP_NORMAL },
+  { N_("Initial step"),STEP_INITIAL },
+  { N_("Macro entry step"),STEP_MACROENTRY },
+  { N_("Macro exit step"),STEP_MACROEXIT },
+  { N_("Macro call step"),STEP_MACROCALL },
+  { N_("Subprogram call step"), STEP_SUBPCALL },
+  { NULL }};
 
+static PropDescription step_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "id", PROP_TYPE_STRING, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_MERGE,
+    N_("Step name"),N_("The name of the step")},
+  { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+    N_("Step type"),N_("The kind of step"),step_style},
+  { "active", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Active"), N_("Shows a red dot to figure the step's activity")},
+  { "font", PROP_TYPE_FONT, PROP_FLAG_VISIBLE,
+    N_("Font"),NULL},
+  { "font_size", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Font size"), NULL, &prop_std_text_height_data},
+  { "font_color", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE,
+    N_("Text colour"), NULL },
+  { "north_pos", PROP_TYPE_POINT, 0},
+  { "south_pos", PROP_TYPE_POINT, 0},
+  PROP_DESC_END
+};
+
+static PropDescription *
+step_describe_props(Step *step) 
+{
+  if (step_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(step_props);
+  }
+  return step_props;
+}    
+
+static PropOffset step_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "id", PROP_TYPE_STRING, offsetof(Step,id)},
+  { "type", PROP_TYPE_ENUM, offsetof(Step,type)},
+  { "active", PROP_TYPE_BOOL, offsetof(Step,active)},
+  { "font", PROP_TYPE_FONT, offsetof(Step,font)},
+  { "font_size", PROP_TYPE_REAL, offsetof(Step,font_size)},
+  { "font_color", PROP_TYPE_COLOUR, offsetof(Step,font_color)},
+  { "north_pos", PROP_TYPE_POINT, offsetof(Step,north.pos)},
+  { "south_pos", PROP_TYPE_POINT, offsetof(Step,south.pos)},  
+  { NULL,0,0 }
+};
+
+static void
+step_get_props(Step *step, GPtrArray *props)
+{  
+  object_get_props_from_offsets(&step->element.object,
+                                step_offsets,props);
+}
+
+static void
+step_set_props(Step *step, GPtrArray *props)
+{
+  object_set_props_from_offsets(&step->element.object,
+                                step_offsets,props);
+  step_been_renamed(step->id);
+  step_update_data(step);
+}
 
 /* the following two functions try to be clever when allocating 
    step numbers */
@@ -229,100 +239,8 @@ static void step_been_renamed(const gchar *sid)
   if (*endptr == '\0') __stepnum = snum + 1; 
 }
 
-static ObjectChange *
-step_apply_properties(Step *step)
-{
-  ObjectState *old_state;
-  StepPropertiesDialog *dlg = step_properties_dialog;
-  
-  PROPDLG_SANITY_CHECK(dlg,step);
-
-  old_state = (ObjectState *)step_get_state(step);
-
-  PROPDLG_APPLY_STRING(dlg,id);
-  PROPDLG_APPLY_BOOL(dlg,active);
-  PROPDLG_APPLY_ENUM(dlg,type);
-  PROPDLG_APPLY_FONT(dlg,font);
-  PROPDLG_APPLY_FONTHEIGHT(dlg,font_size);
-  PROPDLG_APPLY_COLOR(dlg,font_color);
-
-  step_been_renamed(step->id);
-  step_update_data(step);
-  return new_object_state_change(&step->element.object, 
-                                 old_state,
-				 (GetStateFunc)step_get_state,
-				 (SetStateFunc)step_set_state);
-}
-
-PropDlgEnumEntry step_style[] = {
-  { N_("Regular step"),STEP_NORMAL,NULL },
-  { N_("Initial step"),STEP_INITIAL,NULL },
-  { N_("Macro entry step"),STEP_MACROENTRY, NULL },
-  { N_("Macro exit step"),STEP_MACROEXIT, NULL },
-  { N_("Macro call step"),STEP_MACROCALL, NULL },
-  { N_("Subprogram call step"), STEP_SUBPCALL, NULL },
-  { NULL}};
-
-static PROPDLG_TYPE
-step_get_properties(Step *step)
-{
-  StepPropertiesDialog *dlg = step_properties_dialog;
-
-  PROPDLG_CREATE(dlg,step);
-  PROPDLG_SHOW_STRING(dlg,id,_("Step name"));
-  PROPDLG_SHOW_BOOL(dlg,active,_("Active"));
-  PROPDLG_SHOW_ENUM(dlg,type,_("Step type"),step_style);
-  PROPDLG_SHOW_FONT(dlg,font,_("Font:"));
-  PROPDLG_SHOW_FONTHEIGHT(dlg,font_size,_("Font size:"));
-  PROPDLG_SHOW_COLOR(dlg,font_color,_("Text color:"));
-  PROPDLG_READY(dlg);
-  step_properties_dialog = dlg;
-
-  PROPDLG_RETURN(dlg);
-}
-
 static Color color_red = { 1.0f, 0.0f, 0.0f };
 
-static void 
-step_init_defaults(void)
-{
-  static int defaults_initialised = 0;
-  if (!defaults_initialised) {
-    defaults.font = font_getfont(STEP_FONT);
-    defaults.font_size = STEP_FONT_HEIGHT;
-    defaults.font_color = color_black;
-    defaults_initialised = 1;
-  }
-}
-
-static void 
-step_apply_defaults(void)
-{
-  StepDefaultsDialog *dlg = step_defaults_dialog;  
-
-  PROPDLG_APPLY_FONT(dlg,font);
-  PROPDLG_APPLY_FONTHEIGHT(dlg,font_size);
-  PROPDLG_APPLY_COLOR(dlg,font_color);
-}
-
-static PROPDLG_TYPE
-step_get_defaults(void)
-{
-  StepDefaultsDialog *dlg = step_defaults_dialog;
-
-  step_init_defaults();
-  PROPDLG_CREATE(dlg, &defaults);
-  PROPDLG_SHOW_FONT(dlg,font,_("Font:"));
-  PROPDLG_SHOW_FONTHEIGHT(dlg,font_size,_("Font size:"));
-  PROPDLG_SHOW_COLOR(dlg,font_color,_("Color:"));
-  PROPDLG_READY(dlg);
-  step_defaults_dialog = dlg;
-
-  PROPDLG_RETURN(dlg);
-}
-
-
-
 static real
 step_distance_from(Step *step, Point *point)
 {
@@ -353,7 +271,7 @@ step_distance_from(Step *step, Point *point)
 
 static void
 step_select(Step *step, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   element_update_handles(&step->element);
 }
@@ -396,53 +314,54 @@ step_move(Step *step, Point *to)
 
 
 static void
-step_draw(Step *step, Renderer *renderer)
+step_draw(Step *step, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pts[4];
   assert(step != NULL);
   assert(renderer != NULL);
 
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, STEP_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, STEP_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
   pts[0] = step->north.pos;
   pts[1] = step->NU1;
   pts[2] = step->NU2;
   pts[3] = step->A;
-  renderer->ops->draw_polyline(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+  renderer_ops->draw_polyline(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 			       &color_black);
   pts[0] = step->D;
   pts[1] = step->SD1;
   pts[2] = step->SD2;
   pts[3] = step->south.pos;
-  renderer->ops->draw_polyline(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+  renderer_ops->draw_polyline(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 			       &color_black);
 
   if ((step->type == STEP_INITIAL) ||
       (step->type == STEP_MACROCALL) ||
       (step->type == STEP_SUBPCALL)) {
-    renderer->ops->fill_rect(renderer, &step->I, &step->J, &color_white);
-    renderer->ops->draw_rect(renderer, &step->I, &step->J, &color_black);
+    renderer_ops->fill_rect(renderer, &step->I, &step->J, &color_white);
+    renderer_ops->draw_rect(renderer, &step->I, &step->J, &color_black);
   } else {
-    renderer->ops->fill_rect(renderer, &step->E, &step->F, &color_white);
+    renderer_ops->fill_rect(renderer, &step->E, &step->F, &color_white);
   }
-  renderer->ops->draw_rect(renderer, &step->E, &step->F, &color_black);
+  renderer_ops->draw_rect(renderer, &step->E, &step->F, &color_black);
   
   if (step->type != STEP_MACROENTRY)
-    renderer->ops->draw_line(renderer,&step->A,&step->B,&color_black);
+    renderer_ops->draw_line(renderer,&step->A,&step->B,&color_black);
   if (step->type != STEP_MACROEXIT)
-    renderer->ops->draw_line(renderer,&step->C,&step->D,&color_black);
+    renderer_ops->draw_line(renderer,&step->C,&step->D,&color_black);
   
-  renderer->ops->set_font(renderer, step->font, step->font_size);
+  renderer_ops->set_font(renderer, step->font, step->font_size);
 
-  renderer->ops->draw_string(renderer, 
+  renderer_ops->draw_string(renderer, 
 			     step->id, 
 			     &step->G, ALIGN_CENTER, 
 			     &step->font_color);
   if (step->active) 
-    renderer->ops->fill_ellipse(renderer,
+    renderer_ops->fill_ellipse(renderer,
 			       &step->H,
 			       STEP_DOT_RADIUS,STEP_DOT_RADIUS,
 			       &color_red);
@@ -524,14 +443,8 @@ step_update_data(Step *step)
   step->SD1.y = step->SD2.y = (step->south.pos.y + step->D.y) / 2.0;
 
   /* Update connections: */
-#if 0  
-  /* Not really a good idea */
-  step->connections[0].pos = step->north.pos;
-  step->connections[1].pos = step->south.pos;
-#else
   step->connections[0].pos = step->A;
   step->connections[1].pos = step->D;
-#endif
   step->connections[2].pos = step->Z;
   step->connections[3].pos = step->H;
 
@@ -551,38 +464,6 @@ step_update_data(Step *step)
   element_update_handles(elem);
 }
 
-static StepState *
-step_get_state(Step *step)
-{
-  StepState *state = g_new0(StepState, 1);
-  state->obj_state.free = NULL;
-
-  state->id = g_strdup(step->id);
-  state->type = step->type;
-  state->active = step->active;
-  state->font = step->font;
-  state->font_size = step->font_size;
-  state->font_color = step->font_color;
-
-  return state;
-}
-
-static void 
-step_set_state(Step *step, StepState *state)
-{
-  if (step->id) g_free(step->id);
-  step->id = state->id;
-  step->active = state->active;
-  step->type = state->type;
-  step->font = state->font;
-  step->font_size = state->font_size;
-  step->font_color = state->font_color;
-
-  g_free(state);
-
-  step_update_data(step);
-}
-
 static Object *
 step_create(Point *startpoint,
 	      void *user_data,
@@ -595,8 +476,6 @@ step_create(Point *startpoint,
   int i;
   int type;
   
-  step_init_defaults();
-
   step = g_new0(Step,1);
   elem = &step->element;
   obj = &elem->object;
@@ -618,9 +497,9 @@ step_create(Point *startpoint,
 
   step->id = new_step_name();
   step->active = 0;
-  step->font = defaults.font;
-  step->font_size = defaults.font_size;
-  step->font_color = defaults.font_color;
+  step->font = dia_font_new_from_style (STEP_FONT,STEP_FONT_HEIGHT);
+  step->font_size = STEP_FONT_HEIGHT;
+  step->font_color = color_black;
   
   type = GPOINTER_TO_INT(user_data);
   switch(type) {
@@ -660,128 +539,18 @@ step_create(Point *startpoint,
 static void
 step_destroy(Step *step)
 {
-  element_destroy(&step->element);
+  dia_font_unref(step->font);  
   g_free(step->id);
-}
-
-static Object *
-step_copy(Step *step)
-{
-  int i;
-  Step *newstep;
-  Element *elem, *newelem;
-  Object *newobj;
-
-  elem = &step->element;
-  
-  newstep = g_new0(Step,1);
-  newelem = &newstep->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-
-  for (i=0;i<newobj->num_connections;i++) {
-    newobj->connections[i] = &newstep->connections[i];
-    newstep->connections[i].object = newobj;
-    newstep->connections[i].connected = NULL;
-    newstep->connections[i].pos = step->connections[i].pos;
-    newstep->connections[i].last_pos = step->connections[i].last_pos;
-  }
-  newobj->handles[8] = &newstep->north;
-  newobj->handles[9] = &newstep->south;
-  newstep->north.connect_type = HANDLE_CONNECTABLE;
-  newstep->north.type = HANDLE_MAJOR_CONTROL;
-  newstep->north.id = HANDLE_NORTH;
-  newstep->south.connect_type = HANDLE_CONNECTABLE;
-  newstep->south.type = HANDLE_MAJOR_CONTROL;
-  newstep->south.id = HANDLE_SOUTH;
-  newstep->north.pos = newstep->A;
-  newstep->south.pos = newstep->B;
-
-  newstep->id = strdup(step->id);
-  newstep->active = step->active;
-  newstep->type = step->type;
-  newstep->font = step->font;
-  newstep->font_size = step->font_size;
-  newstep->font_color = step->font_color;
-
-  step_update_data(newstep);
-
-  return &newstep->element.object;
-}
-
-static void
-step_save(Step *step, ObjectNode obj_node, const char *filename)
-{
-  element_save(&step->element, obj_node);
-
-  save_enum(obj_node,"type",step->type);
-  save_string(obj_node,"id",step->id);
-  save_boolean(obj_node,"active",step->active);
-  save_font(obj_node,"font",step->font);
-  save_real(obj_node,"font_size",step->font_size);
-  save_color(obj_node,"font_color",&step->font_color);
-  save_point(obj_node,"north_pos",&step->north.pos);
-  save_point(obj_node,"south_pos",&step->south.pos);
+  element_destroy(&step->element);
 }
 
 static Object *
 step_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Step *step;
-  Element *elem;
-  Object *obj;
-  int i;
-  Point pos = { -65536.0, 0.0 };
-
-  step_init_defaults();
-
-  step = g_new0(Step,1);
-  elem = &step->element;
-  obj = &elem->object;
-  
-  obj->type = &step_type;
-  obj->ops = &step_ops;
-
-  element_load(elem, obj_node);
-  
-  step->id = load_string(obj_node,"id",NULL);
-  if (!step->id) step->id = new_step_name();
-  step->type = load_enum(obj_node,"type",STEP_NORMAL);
-  step->active = load_boolean(obj_node,"active",0);
-  step->font = load_font(obj_node,"font",defaults.font);
-  step->font_size = load_real(obj_node,"font_size",defaults.font_size);
-  load_color(obj_node,"font_color",&step->font_color,&defaults.font_color);
-  
-  element_init(elem, 10, 4);
-
-  for (i=0;i<4;i++) {
-    obj->connections[i] = &step->connections[i];
-    step->connections[i].object = obj;
-    step->connections[i].connected = NULL;
-  }
-
-  step->north.connect_type = HANDLE_CONNECTABLE;
-  step->north.type = HANDLE_MAJOR_CONTROL;
-  step->north.id = HANDLE_NORTH;
-  load_point(obj_node,"north_pos",&step->north.pos,&pos);
-
-  step->south.connect_type = HANDLE_CONNECTABLE;
-  step->south.type = HANDLE_MAJOR_CONTROL;
-  step->south.id = HANDLE_SOUTH;
-  load_point(obj_node,"south_pos",&step->south.pos,&pos);
-
-  step_update_data(step);
-
-  for (i=0;i<8;i++) {
-    obj->handles[i]->type = HANDLE_NON_MOVABLE;
-  }
-  obj->handles[8] = &step->north;
-  obj->handles[9] = &step->south;
-
-  return &step->element.object;
+  return object_load_using_properties(&step_type,
+                                      obj_node,version,filename);
 }
-
+ 
 
 
 
