@@ -21,32 +21,48 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
+#include "widgets.h"
 
 #include "uml.h"
 
+#define UMLCLASS_CONNECTIONPOINTS 8
+
 typedef struct _UMLClass UMLClass;
 typedef struct _UMLClassDialog UMLClassDialog;
 
 struct _UMLClass {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[UMLCLASS_CONNECTIONPOINTS];
+
+  /* Class info: */
 
   real font_height;
+  real abstract_font_height;
+  real polymorphic_font_height;
   real classname_font_height;
-  Font *normal_font;
-  Font *abstract_font;
-  Font *classname_font;
-  Font *abstract_classname_font;
+  real abstract_classname_font_height;
+  real comment_font_height;
+
+  DiaFont *normal_font;
+  DiaFont *abstract_font;
+  DiaFont *polymorphic_font;
+  DiaFont *classname_font;
+  DiaFont *abstract_classname_font;
+  DiaFont *comment_font;
   
-  /* Class info: */
   char *name;
   char *stereotype; /* NULL if no stereotype */
+  char *comment; /* Comments on the class */
   int abstract;
-  int suppress_attributes; /* ie. don't draw strings. */
-  int suppress_operations; /* ie. don't draw strings. */
-  int visible_attributes;
+  int suppress_attributes; 
+  int suppress_operations; 
+  int visible_attributes; /* ie. don't draw strings. */
   int visible_operations;
+  int visible_comments;
+
+  Color color_foreground;
+  Color color_background;
 
   /* Attributes: */
   GList *attributes;
@@ -59,18 +75,15 @@ struct _UMLClass {
   GList *formal_params;
 
   /* Calculated variables: */
-  real font_ascent;
   
   real namebox_height;
   char *stereotype_string;
   
   real attributesbox_height;
-  int num_attributes;
-  char **attributes_strings;
+  GList *attributes_strings;
   
   real operationsbox_height;
-  int num_operations;
-  char **operations_strings;
+  GList *operations_strings;
 
   real templates_height;
   real templates_width;
@@ -86,11 +99,28 @@ struct _UMLClassDialog {
 
   GtkEntry *classname;
   GtkEntry *stereotype;
+  GtkEntry *comment;
+
   GtkToggleButton *abstract_class;
   GtkToggleButton *attr_vis;
   GtkToggleButton *attr_supp;
   GtkToggleButton *op_vis;
   GtkToggleButton *op_supp;
+  GtkToggleButton *comments_vis;
+  DiaFontSelector *normal_font;
+  DiaFontSelector *abstract_font;
+  DiaFontSelector *polymorphic_font;
+  DiaFontSelector *classname_font;
+  DiaFontSelector *abstract_classname_font;
+  DiaFontSelector *comment_font;
+  GtkSpinButton *normal_font_height;
+  GtkSpinButton *abstract_font_height;
+  GtkSpinButton *polymorphic_font_height;
+  GtkSpinButton *classname_font_height;
+  GtkSpinButton *abstract_classname_font_height;
+  GtkSpinButton *comment_font_height;
+  DiaColorSelector *fg_color;
+  DiaColorSelector *bg_color;
 
   GList *disconnected_connections;
   GList *added_connections; 
@@ -101,6 +131,7 @@ struct _UMLClassDialog {
   GtkEntry *attr_name;
   GtkEntry *attr_type;
   GtkEntry *attr_value;
+  GtkEntry *attr_comment;
   GtkMenu *attr_visible;
   GtkOptionMenu *attr_visible_button;
   GtkToggleButton *attr_class_scope;
@@ -109,16 +140,24 @@ struct _UMLClassDialog {
   GtkListItem *current_op;
   GtkEntry *op_name;
   GtkEntry *op_type;
+  GtkEntry *op_stereotype;
+  GtkEntry *op_comment;
+
   GtkMenu *op_visible;
   GtkOptionMenu *op_visible_button;
   GtkToggleButton *op_class_scope;
-  GtkToggleButton *op_abstract;
+  GtkMenu *op_inheritance_type;
+  GtkOptionMenu *op_inheritance_type_button;
+  GtkToggleButton *op_query;  
   
   GtkList *parameters_list;
   GtkListItem *current_param;
   GtkEntry *param_name;
   GtkEntry *param_type;
   GtkEntry *param_value;
+  GtkEntry *param_comment;
+  GtkMenu *param_kind;
+  GtkOptionMenu *param_kind_button;
   GtkWidget *param_new_button;
   GtkWidget *param_delete_button;
   GtkWidget *param_up_button;
@@ -131,7 +170,7 @@ struct _UMLClassDialog {
   GtkEntry *templ_type;
 };
 
-extern GtkWidget *umlclass_get_properties(UMLClass *umlclass);
+extern GtkWidget *umlclass_get_properties(UMLClass *umlclass, gboolean is_default);
 extern ObjectChange *umlclass_apply_properties(UMLClass *umlclass);
 extern void umlclass_calculate_data(UMLClass *umlclass);
 extern void umlclass_update_data(UMLClass *umlclass);
