@@ -24,9 +24,11 @@
 #include <glib.h>
 
 #include "geometry.h"
-#include "render.h"
 #include "dia_xml.h"
 #include "object.h"
+#include "text.h"
+#include "intl.h"
+#include "dia_svg.h"
 
 typedef enum {
   GE_LINE,
@@ -35,43 +37,22 @@ typedef enum {
   GE_RECT,
   GE_ELLIPSE,
   GE_PATH,
-  GE_SHAPE
+  GE_SHAPE,
+  GE_TEXT
 } GraphicElementType;
 
 typedef union _GraphicElement GraphicElement;
-typedef struct _GraphicStyle GraphicStyle;
 typedef struct _GraphicElementAny GraphicElementAny;
 typedef struct _GraphicElementLine GraphicElementLine;
 typedef struct _GraphicElementPoly GraphicElementPoly;
 typedef struct _GraphicElementRect GraphicElementRect;
 typedef struct _GraphicElementEllipse GraphicElementEllipse;
 typedef struct _GraphicElementPath GraphicElementPath;
-
-/* special colours */
-#define COLOUR_NONE -1
-#define COLOUR_FOREGROUND -2
-#define COLOUR_BACKGROUND -3
-#define COLOUR_TEXT -4
-
-/* these should be changed if they ever cause a conflict */
-#define LINECAPS_DEFAULT 20
-#define LINEJOIN_DEFAULT 20
-#define LINESTYLE_DEFAULT 20
-
-struct _GraphicStyle {
-  real line_width;
-  gint32 stroke;
-  gint32 fill;
-
-  LineCaps linecap;
-  LineJoin linejoin;
-  LineStyle linestyle;
-  real dashlength;
-};
+typedef struct _GraphicElementText GraphicElementText;
 
 #define SHAPE_INFO_COMMON  \
   GraphicElementType type; \
-  GraphicStyle s
+  DiaSvgGraphicStyle s
 
 struct _GraphicElementAny {
   SHAPE_INFO_COMMON;
@@ -104,6 +85,15 @@ struct _GraphicElementPath {
   int npoints;
   BezPoint points[1];
 };
+
+struct _GraphicElementText {
+  SHAPE_INFO_COMMON;
+  Point anchor;
+  char *string;
+  Text *object;
+  Rectangle text_bounds;
+};
+
 #undef SHAPE_INFO_COMMON
 
 union _GraphicElement {
@@ -116,6 +106,7 @@ union _GraphicElement {
   GraphicElementEllipse ellipse;
   GraphicElementPath path;
   GraphicElementPath shape;
+  GraphicElementText text;
 };
 
 typedef enum {
@@ -136,6 +127,8 @@ struct _ShapeInfo {
   Point *connections;
   Rectangle shape_bounds;
   gboolean has_text;
+  gboolean resize_with_text;
+  gint text_align;
   Rectangle text_bounds;
 
   ShapeAspectType aspect_type;
@@ -146,11 +139,14 @@ struct _ShapeInfo {
   ObjectType *object_type; /* back link so we can find the correct type */
 };
 
+/* there is no destructor for ShapeInfo at the moment */
 ShapeInfo *shape_info_load(const gchar *filename);
 ShapeInfo *shape_info_get(ObjectNode obj_node);
 ShapeInfo *shape_info_getbyname(const gchar *name);
-void shape_info_print(ShapeInfo *info);
 
+void shape_info_realise(ShapeInfo* info);
+void shape_info_print(ShapeInfo *info);
+void parse_path(ShapeInfo *info, const char *path_str, DiaSvgGraphicStyle *s);
 
 #endif
 
