@@ -15,27 +15,31 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "poly_conn.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "diamenu.h"
 #include "message.h"
 #include "properties.h"
 
+#include "create.h"
+
 #include "pixmaps/polyline.xpm"
 
 #define DEFAULT_WIDTH 0.15
 
-
 typedef struct _Polyline {
   PolyConn poly;
 
@@ -51,8 +55,8 @@ static void polyline_move_handle(Polyline *polyline, Handle *handle,
 				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void polyline_move(Polyline *polyline, Point *to);
 static void polyline_select(Polyline *polyline, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void polyline_draw(Polyline *polyline, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void polyline_draw(Polyline *polyline, DiaRenderer *renderer);
 static Object *polyline_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -63,10 +67,8 @@ static void polyline_destroy(Polyline *polyline);
 static Object *polyline_copy(Polyline *polyline);
 
 static PropDescription *polyline_describe_props(Polyline *polyline);
-static void polyline_get_props(Polyline *polyline, Property *props,
-			       guint nprops);
-static void polyline_set_props(Polyline *polyline, Property *props,
-			       guint nprops);
+static void polyline_get_props(Polyline *polyline, GPtrArray *props);
+static void polyline_set_props(Polyline *polyline, GPtrArray *props);
 
 static void polyline_save(Polyline *polyline, ObjectNode obj_node,
 			  const char *filename);
@@ -141,17 +143,17 @@ static PropOffset polyline_offsets[] = {
 };
 
 static void
-polyline_get_props(Polyline *polyline, Property *props, guint nprops)
+polyline_get_props(Polyline *polyline, GPtrArray *props)
 {
-  object_get_props_from_offsets((Object *)polyline, polyline_offsets,
-				props, nprops);
+  object_get_props_from_offsets(&polyline->poly.object, polyline_offsets,
+				props);
 }
 
 static void
-polyline_set_props(Polyline *polyline, Property *props, guint nprops)
+polyline_set_props(Polyline *polyline, GPtrArray *props)
 {
-  object_set_props_from_offsets((Object *)polyline, polyline_offsets,
-				props, nprops);
+  object_set_props_from_offsets(&polyline->poly.object, polyline_offsets,
+				props);
   polyline_update_data(polyline);
 }
 
@@ -173,7 +175,7 @@ static int polyline_closest_segment(Polyline *polyline, Point *point) {
 
 static void
 polyline_select(Polyline *polyline, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   polyconn_update_data(&polyline->poly);
 }
@@ -199,8 +201,9 @@ polyline_move(Polyline *polyline, Point *to)
 }
 
 static void
-polyline_draw(Polyline *polyline, Renderer *renderer)
+polyline_draw(Polyline *polyline, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   PolyConn *poly = &polyline->poly;
   Point *points;
   int n;
@@ -208,30 +211,25 @@ polyline_draw(Polyline *polyline, Renderer *renderer)
   points = &poly->points[0];
   n = poly->numpoints;
 
-  renderer->ops->set_linewidth(renderer, polyline->line_width);
-  renderer->ops->set_linestyle(renderer, polyline->line_style);
-  renderer->ops->set_dashlength(renderer, polyline->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, polyline->line_width);
+  renderer_ops->set_linestyle(renderer, polyline->line_style);
+  renderer_ops->set_dashlength(renderer, polyline->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_polyline(renderer, points, n, &polyline->line_color);
-
-  if (polyline->start_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, polyline->start_arrow.type,
-	       &points[0], &points[1],
-	       polyline->start_arrow.length, polyline->start_arrow.width,
-	       polyline->line_width,
-	       &polyline->line_color, &color_white);
-  }
-  if (polyline->end_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, polyline->end_arrow.type,
-	       &points[n-1], &points[n-2],
-	       polyline->end_arrow.length, polyline->end_arrow.width,
-	       polyline->line_width,
-	       &polyline->line_color, &color_white);
-  }
+  renderer_ops->draw_polyline_with_arrows(renderer,
+					   points, n,
+					   polyline->line_width,
+					   &polyline->line_color,
+					   &polyline->start_arrow,
+					   &polyline->end_arrow);
 }
 
+/** user_data is a struct polyline_create_data, containing an array of 
+    points and a count.
+    If user_data is NULL, the startpoint is used and a 1x1 line is created.
+    Otherwise, the startpoint is ignored.
+*/
 static Object *
 polyline_create(Point *startpoint,
 		  void *user_data,
@@ -244,19 +242,35 @@ polyline_create(Point *startpoint,
   Point defaultlen = { 1.0, 1.0 };
 
   /*polyline_init_defaults();*/
-  polyline = g_malloc(sizeof(Polyline));
+  polyline = g_malloc0(sizeof(Polyline));
   poly = &polyline->poly;
-  obj = (Object *) polyline;
+  obj = &poly->object;
   
   obj->type = &polyline_type;
   obj->ops = &polyline_ops;
 
-  polyconn_init(poly);
+  if (user_data == NULL) {
+    polyconn_init(poly, 2);
 
-  poly->points[0] = *startpoint;
-  poly->points[1] = *startpoint;
-  point_add(&poly->points[1], &defaultlen);
+    poly->points[0] = *startpoint;
+    poly->points[1] = *startpoint;
 
+    point_add(&poly->points[1], &defaultlen);
+
+    *handle1 = poly->object.handles[0];
+    *handle2 = poly->object.handles[1];
+  } else {
+    MultipointCreateData *pcd = (MultipointCreateData *)user_data;
+
+    polyconn_init(poly, pcd->num_points);
+
+    /* Handles are set up by polyconn_init and polyconn_update_data */
+    polyconn_set_points(poly, pcd->num_points, pcd->points);
+
+    *handle1 = poly->object.handles[0];
+    *handle2 = poly->object.handles[pcd->num_points-1];
+  }
+  
   polyline_update_data(polyline);
 
   polyline->line_width =  attributes_get_default_linewidth();
@@ -266,9 +280,7 @@ polyline_create(Point *startpoint,
   polyline->start_arrow = attributes_get_default_start_arrow();
   polyline->end_arrow = attributes_get_default_end_arrow();
 
-  *handle1 = poly->object.handles[0];
-  *handle2 = poly->object.handles[1];
-  return (Object *)polyline;
+  return &polyline->poly.object;
 }
 
 static void
@@ -286,9 +298,9 @@ polyline_copy(Polyline *polyline)
   
   poly = &polyline->poly;
  
-  newpolyline = g_malloc(sizeof(Polyline));
+  newpolyline = g_malloc0(sizeof(Polyline));
   newpoly = &newpolyline->poly;
-  newobj = (Object *) newpolyline;
+  newobj = &newpoly->object;
 
   polyconn_copy(poly, newpoly);
 
@@ -299,39 +311,31 @@ polyline_copy(Polyline *polyline)
   newpolyline->start_arrow = polyline->start_arrow;
   newpolyline->end_arrow = polyline->end_arrow;
 
-  return (Object *)newpolyline;
+  return &newpolyline->poly.object;
 }
 
-
 static void
 polyline_update_data(Polyline *polyline)
 {
   PolyConn *poly = &polyline->poly;
-  Object *obj = (Object *) polyline;
+  Object *obj = &poly->object;
+  PolyBBExtras *extra = &poly->extra_spacing;
 
   polyconn_update_data(&polyline->poly);
+
+  extra->start_trans =  (polyline->line_width / 2.0);
+  extra->end_trans =     (polyline->line_width / 2.0);
+  extra->middle_trans = (polyline->line_width / 2.0);
     
+  if (polyline->start_arrow.type != ARROW_NONE) 
+    extra->start_trans = MAX(extra->start_trans,polyline->start_arrow.width);
+  if (polyline->end_arrow.type != ARROW_NONE) 
+    extra->end_trans = MAX(extra->end_trans,polyline->end_arrow.width);
+
+  extra->start_long = (polyline->line_width / 2.0);
+  extra->end_long   = (polyline->line_width / 2.0);
+
   polyconn_update_boundingbox(poly);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= polyline->line_width/2;
-  obj->bounding_box.left -= polyline->line_width/2;
-  obj->bounding_box.bottom += polyline->line_width/2;
-  obj->bounding_box.right += polyline->line_width/2;
-
-  /* Fix boundingbox for arrowheads */
-  if (polyline->start_arrow.type != ARROW_NONE ||
-      polyline->end_arrow.type != ARROW_NONE) {
-    real arrow_width = 0.0;
-    if (polyline->start_arrow.type != ARROW_NONE)
-      arrow_width = polyline->start_arrow.width;
-    if (polyline->end_arrow.type != ARROW_NONE)
-      arrow_width = MAX(arrow_width, polyline->start_arrow.width);
-
-    obj->bounding_box.top -= arrow_width;
-    obj->bounding_box.left -= arrow_width;
-    obj->bounding_box.bottom += arrow_width;
-    obj->bounding_box.right += arrow_width;
-  }
 
   obj->position = poly->points[0];
 }
@@ -386,10 +390,10 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   AttributeNode attr;
 
-  polyline = g_malloc(sizeof(Polyline));
+  polyline = g_malloc0(sizeof(Polyline));
 
   poly = &polyline->poly;
-  obj = (Object *) polyline;
+  obj = &poly->object;
   
   obj->type = &polyline_type;
   obj->ops = &polyline_ops;
@@ -444,7 +448,7 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
 
   polyline_update_data(polyline);
 
-  return (Object *)polyline;
+  return &polyline->poly.object;
 }
 
 static ObjectChange *
