@@ -43,12 +43,13 @@
 #include "intl.h"
 #include "message.h"
 #include "geometry.h"
-#include "render.h"
 #include "filter.h"
 #include "object.h"
 #include "properties.h"
-#include "../app/group.h"
+#include "propinternals.h"
+#include "group.h"
 
+#include "create.h"
 #include "xfig.h"
 
 #define BUFLEN 512
@@ -57,32 +58,6 @@ static Color fig_colors[FIG_MAX_USER_COLORS];
 
 gboolean import_fig(const gchar *filename, DiagramData *dia, void* user_data);
 
-static char **warnings;
-
-#define WARNING_NO_POLYGONS 0
-#define WARNING_NO_PATTERNS 1
-#define WARNING_NO_TRIPLE_DOTS 2
-#define WARNING_NEGATIVE_CORNER_RADIUS 3
-#define WARNING_NO_SPLINES 4
-#define MAX_WARNING 5
-
-static void
-fig_warn(int warning) {
-    if (warnings == NULL) {
-	warnings = g_malloc(sizeof(char*)*MAX_WARNING);
-	warnings[0] = _("Polygon import is not implemented yes");
-	warnings[1] = _("Patterns are not supported by Dia");
-	warnings[2] = _("Triple-dotted lines are not supported by Dia, using double-dotted");
-	warnings[3] = _("Negative corner radius, negating");
-	warnings[4] = _("Spline import is not implemented yet");
-    }
-    if (warning >= MAX_WARNING) return;
-    if (warnings[warning] != NULL) {
-	message_warning(warnings[warning]);
-	warnings[warning] = NULL;
-    }
-}
-
 static int
 skip_comments(FILE *file) {
   int ch;
@@ -111,57 +86,79 @@ skip_comments(FILE *file) {
 }
 
 static Object *
-create_standard_text(real xpos, real ypos, char *text,
+create_standard_text(real xpos, real ypos,
 		     DiagramData *dia) {
     ObjectType *otype = object_get_type("Standard - Text");
     Object *new_obj;
     Handle *h1, *h2;
     Point point;
-    Property props[4];
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
 
     point.x = xpos;
     point.y = ypos;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    //    layer_add_object(dia->active_layer, new_obj);
-
-    props[0].name = "text";
-    props[0].type = PROP_TYPE_STRING;
-    PROP_VALUE_STRING(props[0]) = text;
-    new_obj->ops->set_props(new_obj, props, 1);
-
+    
     return new_obj;
 }
 
+static PropDescription xfig_element_prop_descs[] = {
+    { "elem_corner", PROP_TYPE_POINT },
+    { "elem_width", PROP_TYPE_REAL },
+    { "elem_height", PROP_TYPE_REAL },
+    PROP_DESC_END};
+
+static GPtrArray *make_element_props(real xpos, real ypos,
+                                     real width, real height) 
+{
+    GPtrArray *props;
+    PointProperty *pprop;
+    RealProperty *rprop;
+
+    props = prop_list_from_descs(xfig_element_prop_descs,pdtpp_true);
+    g_assert(props->len == 3);
+    
+    pprop = g_ptr_array_index(props,0);
+    pprop->point_data.x = xpos;
+    pprop->point_data.y = ypos;
+    rprop = g_ptr_array_index(props,1);
+    rprop->real_data = width;
+    rprop = g_ptr_array_index(props,2);
+    rprop->real_data = height;
+    
+    return props;
+}
+
 static Object *
 create_standard_ellipse(real xpos, real ypos, real width, real height,
 			DiagramData *dia) {
     ObjectType *otype = object_get_type("Standard - Ellipse");
     Object *new_obj;
     Handle *h1, *h2;
-    Property props[4];
+    
+    GPtrArray *props;
     Point point;
 
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
+
     point.x = xpos;
     point.y = ypos;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    //    layer_add_object(dia->active_layer, new_obj);
+    /*   layer_add_object(dia->active_layer, new_obj); */
   
-    props[0].name = "elem_corner";
-    props[0].type = PROP_TYPE_POINT;
-    PROP_VALUE_POINT(props[0]).x = xpos;
-    PROP_VALUE_POINT(props[0]).y = ypos;
-    props[1].name = "elem_width";
-    props[1].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[1]) = width;
-    props[2].name = "elem_height";
-    props[2].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[2]) = height;
-
-    new_obj->ops->set_props(new_obj, props, 3);
+    props = make_element_props(xpos,ypos,width,height);
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
 
     return new_obj;
 }
@@ -174,31 +171,135 @@ create_standard_box(real xpos, real ypos, real width, real height,
   Object *new_obj;
   Handle *h1, *h2;
   Point point;
-  Property props[4];
+  GPtrArray *props;
+
+  if (otype == NULL){
+      message_error(_("Can't find standard object"));
+      return NULL;
+  }
 
   point.x = xpos;
   point.y = ypos;
 
   new_obj = otype->ops->create(&point, otype->default_user_data,
 			       &h1, &h2);
-  //  layer_add_object(dia->active_layer, new_obj);
+  /*  layer_add_object(dia->active_layer, new_obj); */
   
-  props[0].name = "elem_corner";
-  props[0].type = PROP_TYPE_POINT;
-  PROP_VALUE_POINT(props[0]).x = xpos;
-  PROP_VALUE_POINT(props[0]).y = ypos;
-  props[1].name = "elem_width";
-  props[1].type = PROP_TYPE_REAL;
-  PROP_VALUE_REAL(props[1]) = width;
-  props[2].name = "elem_height";
-  props[2].type = PROP_TYPE_REAL;
-  PROP_VALUE_REAL(props[2]) = height;
-
-  new_obj->ops->set_props(new_obj, props, 3);
+  props = make_element_props(xpos,ypos,width,height);
+  new_obj->ops->set_props(new_obj, props);
+  prop_list_free(props);
 
   return new_obj;
 }
 
+static Object *
+create_standard_polyline(int num_points, 
+			 Point *points,
+			 DiagramData *dia) {
+    ObjectType *otype = object_get_type("Standard - PolyLine");
+    Object *new_obj;
+    Handle *h1, *h2;
+    MultipointCreateData *pcd;
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
+
+    pcd = g_new(MultipointCreateData, 1);
+    pcd->num_points = num_points;
+    pcd->points = points;
+
+    new_obj = otype->ops->create(NULL, pcd,
+				 &h1, &h2);
+
+    g_free(pcd);
+    
+    return new_obj;
+}
+
+static Object *
+create_standard_polygon(int num_points, 
+			Point *points,
+			DiagramData *dia) {
+    ObjectType *otype = object_get_type("Standard - Polygon");
+    Object *new_obj;
+    Handle *h1, *h2;
+    MultipointCreateData *pcd;
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
+
+    pcd = g_new(MultipointCreateData, 1);
+    pcd->num_points = num_points;
+    pcd->points = points;
+
+    new_obj = otype->ops->create(NULL, pcd, &h1, &h2);
+
+    g_free(pcd);
+    
+    return new_obj;
+}
+
+static Object *
+create_standard_bezierline(int num_points, 
+			   BezPoint *points,
+			   DiagramData *dia) {
+    ObjectType *otype = object_get_type("Standard - BezierLine");
+    Object *new_obj;
+    Handle *h1, *h2;
+    BezierCreateData *bcd;
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
+
+    bcd = g_new(BezierCreateData, 1);
+    bcd->num_points = num_points;
+    bcd->points = points;
+
+    new_obj = otype->ops->create(NULL, bcd,
+				 &h1, &h2);
+
+    g_free(bcd);
+    
+    return new_obj;
+}
+
+static Object *
+create_standard_beziergon(int num_points, 
+			  BezPoint *points,
+			  DiagramData *dia) {
+    ObjectType *otype = object_get_type("Standard - Beziergon");
+    Object *new_obj;
+    Handle *h1, *h2;
+    BezierCreateData *bcd;
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
+
+    bcd = g_new(BezierCreateData, 1);
+    bcd->num_points = num_points;
+    bcd->points = points;
+
+    new_obj = otype->ops->create(NULL, bcd,
+				 &h1, &h2);
+
+    g_free(bcd);
+    
+    return new_obj;
+}
+
+
+static PropDescription xfig_arc_prop_descs[] = {
+    { "curve_distance", PROP_TYPE_REAL },
+    PROP_DESC_END};
+
 static Object *
 create_standard_arc(real x1, real y1, real x2, real y2,
 		    real radius, DiagramData *dia) {
@@ -206,40 +307,34 @@ create_standard_arc(real x1, real y1, real x2, real y2,
     Object *new_obj;
     Handle *h1, *h2;
     Point point;
-    Property props[4];
+    GPtrArray *props;
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
 
     point.x = x1;
     point.y = y1;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    //    layer_add_object(dia->active_layer, new_obj);
-
+    /*    layer_add_object(dia->active_layer, new_obj); */
     
-    /*
-    props[0].name = "elem_corner";
-    props[0].type = PROP_TYPE_POINT;
-    PROP_VALUE_POINT(props[0]).x = xpos;
-    PROP_VALUE_POINT(props[0]).y = ypos;
-    props[1].name = "elem_width";
-    props[1].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[1]) = width;
-    props[2].name = "elem_height";
-    props[2].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[2]) = height;
-    props[3].name = "image_file";
-    props[3].type = PROP_TYPE_FILE;
-    PROP_VALUE_FILE(props[3]) = file;
-    */
-    props[0].name = "curve_distance";
-    props[0].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[0]) = radius;
-
-    new_obj->ops->set_props(new_obj, props, 1);
+    props = prop_list_from_descs(xfig_arc_prop_descs,pdtpp_true);
+    g_assert(props->len == 1);
+    
+    ((RealProperty *)g_ptr_array_index(props,0))->real_data = radius;
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
 
     return new_obj;
 }
 
+static PropDescription xfig_file_prop_descs[] = {
+    { "image_file", PROP_TYPE_FILE },
+    PROP_DESC_END};
+
 static Object *
 create_standard_image(real xpos, real ypos, real width, real height,
 		      char *file, DiagramData *dia) {
@@ -247,29 +342,33 @@ create_standard_image(real xpos, real ypos, real width, real height,
     Object *new_obj;
     Handle *h1, *h2;
     Point point;
-    Property props[4];
+    GPtrArray *props;
+    StringProperty *sprop;
+
+    if (otype == NULL){
+	message_error(_("Can't find standard object"));
+	return NULL;
+    }
 
     point.x = xpos;
     point.y = ypos;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    //    layer_add_object(dia->active_layer, new_obj);
+    /*    layer_add_object(dia->active_layer, new_obj); */
     
-    props[0].name = "elem_corner";
-    props[0].type = PROP_TYPE_POINT;
-    PROP_VALUE_POINT(props[0]).x = xpos;
-    PROP_VALUE_POINT(props[0]).y = ypos;
-    props[1].name = "elem_width";
-    props[1].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[1]) = width;
-    props[2].name = "elem_height";
-    props[2].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[2]) = height;
-    props[3].name = "image_file";
-    props[3].type = PROP_TYPE_FILE;
-    PROP_VALUE_FILE(props[3]) = file;
-    new_obj->ops->set_props(new_obj, props, 4);
+    props = make_element_props(xpos,ypos,width,height);
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
+
+
+    props = prop_list_from_descs(xfig_file_prop_descs,pdtpp_true);
+    g_assert(props->len == 1);    
+    sprop = g_ptr_array_index(props,0);
+    g_free(sprop->string_data);
+    sprop->string_data = g_strdup(file);
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
 
     return new_obj;
 }
@@ -280,16 +379,19 @@ create_standard_group(GList *items, DiagramData *dia) {
 
     new_obj = group_create((GList*)items);
 
-    //    layer_add_object(dia->active_layer, new_obj);
+    /*    layer_add_object(dia->active_layer, new_obj); */
 
     return new_obj;
 }
 
 static Color
-fig_color(int color_index) {
-    if (color_index == -1) return color_black; /* Default color */
-    if (color_index < 32) return fig_default_colors[color_index];
-    else return fig_colors[color_index-32];
+fig_color(int color_index) 
+{
+    if (color_index == -1) 
+        return color_black; /* Default color */
+    if (color_index < FIG_MAX_DEFAULT_COLORS) 
+        return fig_default_colors[color_index];
+    else return fig_colors[color_index-FIG_MAX_DEFAULT_COLORS];
 }
 
 static Color
@@ -313,12 +415,43 @@ fig_area_fill_color(int area_fill, int color_index) {
 	col.green += (0xff-col.green)*(area_fill-20)/20;
 	col.blue += (0xff-col.blue)*(area_fill-20)/20;
     } else {
-	fig_warn(WARNING_NO_PATTERNS);
+	message_warning(_("Patterns are not supported by Dia"));
     }
     
     return col;
 }
 
+
+
+static PropDescription xfig_simple_prop_descs_line[] = {
+    { "line_width", PROP_TYPE_REAL },
+    { "line_colour", PROP_TYPE_COLOUR },
+    PROP_DESC_END};
+
+static LineStyle 
+fig_line_style_to_dia(int line_style) 
+{
+    switch (line_style) {
+    case 0:
+        return LINESTYLE_SOLID;
+    case 1:
+        return LINESTYLE_DASHED;
+    case 2:
+	return LINESTYLE_DOTTED;
+    case 3:
+        return LINESTYLE_DASH_DOT;
+    case 4:
+        return LINESTYLE_DASH_DOT_DOT;
+    case 5:
+        message_warning(_("Triple-dotted lines are not supported by Dia, "
+			  "using double-dotted"));
+        return LINESTYLE_DASH_DOT_DOT;
+    default:
+        message_error(_("Line style %d should not appear\n"), line_style);
+        return LINESTYLE_SOLID;
+    }
+}
+
 static void
 fig_simple_properties(Object *obj,
 		      int line_style,
@@ -326,60 +459,50 @@ fig_simple_properties(Object *obj,
 		      int pen_color,
 		      int fill_color,
 		      int area_fill) {
-    Property props[5];
-    int num_props = 0;
+    GPtrArray *props = prop_list_from_descs(xfig_simple_prop_descs_line,
+                                            pdtpp_true);
+    RealProperty *rprop;
+    ColorProperty *cprop;
+
+    g_assert(props->len == 2);
+    
+    rprop = g_ptr_array_index(props,0);
+    rprop->real_data = thickness/FIG_ALT_UNIT;
+    
+    cprop = g_ptr_array_index(props,1);
+    cprop->color_data = fig_color(pen_color);
+
 
-    num_props = 0;
     if (line_style != -1) {
-	props[num_props].name = "line_style";
-	props[num_props].type = PROP_TYPE_ENUM;
-	switch (line_style) {
-	case 0:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_SOLID;
-	    break;
-	case 1:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASHED;
-	    break;
-	case 2:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DOTTED;
-	    break;
-	case 3:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASH_DOT;
-	    break;
-	case 4:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASH_DOT_DOT;
-	    break;
-	case 5:
-	    fig_warn(WARNING_NO_TRIPLE_DOTS);
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASH_DOT_DOT;
-	    break;
-	default:
-	    message_error(_("Line style %d should not appear\n"), line_style);
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_SOLID;
-	}
-	num_props++;
-    }
-    props[num_props].name = "line_width";
-    props[num_props].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[num_props]) = thickness/FIG_ALT_UNIT;
-    num_props++;
-    props[num_props].name = "line_colour";
-    props[num_props].type = PROP_TYPE_COLOUR;
-    PROP_VALUE_COLOUR(props[num_props]) = fig_color(pen_color);
-    num_props++;
-    if (area_fill == -1) {
-	props[num_props].name = "show_background";
-	props[num_props].type = PROP_TYPE_BOOL;
-	PROP_VALUE_BOOL(props[num_props]) = FALSE;
-	num_props++;
-    } else {
-	props[num_props].name = "fill_colour";
-	props[num_props].type = PROP_TYPE_COLOUR;
-	PROP_VALUE_COLOUR(props[num_props]) = fig_area_fill_color(area_fill, fill_color);
-	num_props++;
+        LinestyleProperty *lsprop = 
+            (LinestyleProperty *)make_new_prop("line_style", 
+                                               PROP_TYPE_LINESTYLE,
+                                               PROP_FLAG_DONT_SAVE);
+        lsprop->dash = 1.0;
+        lsprop->style = fig_line_style_to_dia(line_style);
+
+        g_ptr_array_add(props,lsprop);
     }
 
-    obj->ops->set_props(obj, props, num_props);
+    if (area_fill == -1) {
+        BoolProperty *bprop = 
+            (BoolProperty *)make_new_prop("show_background",
+                                          PROP_TYPE_BOOL,PROP_FLAG_DONT_SAVE);
+        bprop->bool_data = FALSE;
+
+        g_ptr_array_add(props,bprop);
+    } else {
+        ColorProperty *cprop = 
+            (ColorProperty *)make_new_prop("fill_colour",
+                                           PROP_TYPE_COLOUR,
+                                           PROP_FLAG_DONT_SAVE);
+        cprop->color_data = fig_area_fill_color(area_fill, fill_color);
+
+        g_ptr_array_add(props,cprop);
+    }
+
+    obj->ops->set_props(obj, props);
+    prop_list_free(props);
 }
 
 static int
@@ -513,7 +636,7 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
 				     (2*radius_x)/FIG_UNIT,
 				     (2*radius_y)/FIG_UNIT,
 				     dia);
-
+    if (newobj == NULL) return NULL;
     fig_simple_properties(newobj, line_style, thickness,
 			  pen_color, fill_color, area_fill);
 
@@ -533,135 +656,356 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
 
 static Object *
 fig_read_polyline(FILE *file, DiagramData *dia) {
-     int sub_type;
-     int line_style;
-     int thickness;
-     int pen_color;
-     int fill_color;
-     int depth;
-     int pen_style;
-     int area_fill;
-     real style_val;
-     int join_style;
-     int cap_style;
-     int radius;
-     int forward_arrow;
-     int backward_arrow;
-     int npoints;
-     Point *points;
-     Property props[5];
-     int num_props;
-     Object *newobj = NULL;
-     int flipped = 0;
-     char *image_file = NULL;
+    int sub_type;
+    int line_style;
+    int thickness;
+    int pen_color;
+    int fill_color;
+    int depth;
+    int pen_style;
+    int area_fill;
+    real style_val;
+    int join_style;
+    int cap_style;
+    int radius;
+    int forward_arrow;
+    int backward_arrow;
+    int npoints;
+    Point *points;
+    GPtrArray *props = g_ptr_array_new();
+    Object *newobj = NULL;
+    int flipped = 0;
+    char *image_file = NULL;
 
-     if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d %d %d\n",
-		&sub_type,
-		&line_style,
-		&thickness,
-		&pen_color,
-		&fill_color,
-		&depth,
-		&pen_style,
-		&area_fill,
-		&style_val,
-		&join_style,
-		&cap_style,
-		&radius,
-		&forward_arrow,
-		&backward_arrow,
-		&npoints) != 15) {
-       message_error(_("Couldn't read polyline info: %s\n"), strerror(errno));
-       return NULL;
-     }
+    if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d %d %d\n",
+	       &sub_type,
+	       &line_style,
+	       &thickness,
+	       &pen_color,
+	       &fill_color,
+	       &depth,
+	       &pen_style,
+	       &area_fill,
+	       &style_val,
+	       &join_style,
+	       &cap_style,
+	       &radius,
+	       &forward_arrow,
+	       &backward_arrow,
+	       &npoints) != 15) {
+	message_error(_("Couldn't read polyline info: %s\n"), strerror(errno));
+	return NULL;
+    }
 
-     if (forward_arrow == 1) {
-       fig_read_arrow(file);
-     }
+    if (forward_arrow == 1) {
+	fig_read_arrow(file);
+    }
 
-     if (backward_arrow == 1) {
-       fig_read_arrow(file);
-     }
+    if (backward_arrow == 1) {
+	fig_read_arrow(file);
+    }
 
-     if (sub_type == 5) { /* image has image name before npoints */
-	                  /* Despite what the specs say */
-	 if (fscanf(file, " %d", &flipped) != 1) {
-	     message_error(_("Couldn't read flipped bit: %s\n"), strerror(errno));
-	     return NULL;
-	 }
+    if (sub_type == 5) { /* image has image name before npoints */
+	/* Despite what the specs say */
+	if (fscanf(file, " %d", &flipped) != 1) {
+	    message_error(_("Couldn't read flipped bit: %s\n"), strerror(errno));
+	    return NULL;
+	}
 
-	 image_file = fig_read_text_line(file);
+	image_file = fig_read_text_line(file);
 
-     }
+    }
 
-     if (!fig_read_n_points(file, npoints, &points)) {
-       return NULL;
-     }
+    if (!fig_read_n_points(file, npoints, &points)) {
+	return NULL;
+    }
      
-     num_props = 0;
-     switch (sub_type) {
-     case 4:
-	 if (radius < 0) {
-	     fig_warn(WARNING_NEGATIVE_CORNER_RADIUS);
-  	     props[0].name = "corner_radius";
-	     props[0].type = PROP_TYPE_REAL;
-	     PROP_VALUE_REAL(props[0]) = -radius/FIG_ALT_UNIT;
-	     num_props++;
-	 } else {
-	     props[0].name = "corner_radius";
-	     props[0].type = PROP_TYPE_REAL;
-	     PROP_VALUE_REAL(props[0]) = radius/FIG_ALT_UNIT;
-	     num_props++;
-	 }
-	 /* Notice fallthrough */
-     case 2: /* box */
-	 if (points[0].x > points[2].x) {
-	     real tmp = points[0].x;
-	     points[0].x = points[2].x;
-	     points[2].x = tmp;
-	 }
-	 if (points[0].y > points[2].y) {
-	     real tmp = points[0].y;
-	     points[0].y = points[2].y;
-	     points[2].y = tmp;
-	 }
-	 newobj = create_standard_box(points[0].x, points[0].y,
-				      points[2].x-points[0].x,
-				      points[2].y-points[0].y, dia);
-	 newobj->ops->set_props(newobj, props, num_props);
-	 break;
-     case 5: /* imported-picture bounding-box) */
-	 newobj = create_standard_image(points[0].x, points[0].y,
-					points[2].x-points[0].x,
-					points[2].y-points[0].y,
-					image_file, dia);
-	 break;
-     case 1: /* polyline */
-     case 3: /* polygon */
-	 /*
-	   newobj = create_standard_polygon(points, dia);
-	 */
-     default: 
-       message_error(_("Unknown polyline subtype: %d\n"), sub_type);
-       return NULL;
-     }
-     if (image_file != NULL)
-	 g_free(image_file);
+    switch (sub_type) {
+    case 4: {
+	RealProperty *rprop = 
+	    (RealProperty *)make_new_prop("corner_radius",
+					  PROP_TYPE_REAL,PROP_FLAG_DONT_SAVE);
+	if (radius < 0) {
+	    message_warning(_("Negative corner radius, negating"));
+	    rprop->real_data = -radius/FIG_ALT_UNIT;
+	} else {
+	    rprop->real_data = radius/FIG_ALT_UNIT;
+	}
+	g_ptr_array_add(props,rprop);
+    }
+	/* Notice fallthrough */
+    case 2: /* box */
+	if (points[0].x > points[2].x) {
+	    real tmp = points[0].x;
+	    points[0].x = points[2].x;
+	    points[2].x = tmp;
+	}
+	if (points[0].y > points[2].y) {
+	    real tmp = points[0].y;
+	    points[0].y = points[2].y;
+	    points[2].y = tmp;
+	}
+	newobj = create_standard_box(points[0].x, points[0].y,
+				     points[2].x-points[0].x,
+				     points[2].y-points[0].y, dia);
+	if (newobj == NULL) goto exit;
+	newobj->ops->set_props(newobj, props);
+	break;
+    case 5: /* imported-picture bounding-box) */
+	newobj = create_standard_image(points[0].x, points[0].y,
+				       points[2].x-points[0].x,
+				       points[2].y-points[0].y,
+				       image_file, dia);
+	if (newobj == NULL) goto exit;
+	break;
+    case 1: /* polyline */
+	newobj = create_standard_polyline(npoints, points, dia);
+	if (newobj == NULL) goto exit;
+	break;
+    case 3: /* polygon */
+	newobj = create_standard_polygon(npoints, points, dia);
+	if (newobj == NULL) goto exit;
+	break;
+    default: 
+	message_error(_("Unknown polyline subtype: %d\n"), sub_type);
+	goto exit;
+    }
 
-     fig_simple_properties(newobj, line_style, thickness,
-			   pen_color, fill_color, area_fill);
-     /* Pen style field (not used) */
-     /* Style_val (size of dots and dashes) in 1/80 inch*/
-     /* Join style */
-     /* Cap style */
+    fig_simple_properties(newobj, line_style, thickness,
+			  pen_color, fill_color, area_fill);
+    /* Pen style field (not used) */
+    /* Style_val (size of dots and dashes) in 1/80 inch*/
+    /* Join style */
+    /* Cap style */
      
-     /* Depth field */
-     if (compound_stack == NULL)
-	 depths[depth] = g_list_prepend(depths[depth], newobj);
-     else
-	 if (compound_depth > depth) compound_depth = depth;
+    /* Depth field */
+    if (compound_stack == NULL)
+	depths[depth] = g_list_prepend(depths[depth], newobj);
+    else
+	if (compound_depth > depth) compound_depth = depth;
+ exit:
+    prop_list_free(props);
+    if (image_file != NULL)
+	g_free(image_file);
+    return newobj;
+}
 
-     return newobj;
+#define TENSION 0.25
+
+static BezPoint *transform_spline(int npoints, Point *points, gboolean closed) {
+    BezPoint *bezpoints = g_new(BezPoint, npoints);
+    int i;
+    Point vector;
+
+    for (i = 0; i < npoints; i++) {
+	bezpoints[i].p3 = points[i];
+	bezpoints[i].type = BEZ_CURVE_TO;
+    }
+    bezpoints[0].type = BEZ_MOVE_TO;
+    bezpoints[0].p1 = points[0];
+    for (i = 1; i < npoints-1; i++) {
+	bezpoints[i].p2 = points[i];
+	bezpoints[i+1].p1 = points[i];
+	vector = points[i-1];
+	point_sub(&vector, &points[i+1]);
+	point_scale(&vector, -TENSION);
+	point_sub(&bezpoints[i].p2, &vector);
+	point_add(&bezpoints[i+1].p1, &vector);
+    }
+    if (closed) {
+	bezpoints[npoints-1].p2 = points[i];
+	bezpoints[1].p1 = points[i];
+	vector = points[npoints-2];
+	point_sub(&vector, &points[1]);
+	point_scale(&vector, -TENSION);
+	point_sub(&bezpoints[npoints-1].p2, &vector);
+	point_add(&bezpoints[1].p1, &vector);
+    } else {
+	bezpoints[1].p1 = points[0];
+	bezpoints[npoints-1].p2 = bezpoints[npoints-1].p3;
+    }
+    return bezpoints;
+}
+
+/* The XFig 'X-Splines' seem to be a generalization of cubic B-Splines
+ * and Catmull-Rom splines.
+ * Our Beziers, if Beziers they are, have the basis matrix M_bez
+ * [-1  3 -3  1]
+ * [ 3 -6  3  0]
+ * [-3  3  0  0]
+ * [ 1  0  0  0]
+ *
+ * The basis matrix for cubic B-Splines according to Hearn & Baker (M_b)
+ * [-1  3 -3  1]
+ * [ 3 -6  3  0]*1/6
+ * [-3  0  3  0]
+ * [ 1  4  1  0]
+ * So the transformation matrix (M_bez^-1)*M_b should be
+ * [ 1  4  1  0]
+ * [ 0  4  2  0]*1/6
+ * [ 0  2  4  0]
+ * [ 0  1  4  1] 
+ *
+ * The basis matrix for Catmull-Rom splines (M_c) is:
+ * [-s 2-s s-2 s]
+ * [2s s-3 3-2s -s]
+ * [-s  0  s  0]
+ * [ 0  1  0  0] where s = (1-t)/2 and t = 0, so s = 1/2:
+ * [ -.5  1.5 -1.5   .5] 
+ * [   1 -2.5    2  -.5] 
+ * [ -.5    0   .5    0] 
+ * [   0    1    4    1] 
+ * Thus, the transformation matrix for the interpolated splines should be
+ * (M_bez^-1)*M_c
+ * The conversion matrix should then be:
+ * [0    1    4    1]
+ * [-1/6 1 4+1/6   1]
+ * [0   1/6   5  5/6]
+ * [0    0    5    1]
+ * or
+ * [ 0  6  24  6]
+ * [-1  6  25  6]*1/6
+ * [ 0  1  30  5]
+ * [ 0  0  30  5]
+ */
+
+static real matrix_bspline_to_bezier[4][4] =
+    {{1/6.0, 4/6.0, 1/6.0, 0},
+     {0,   4/6.0, 2/6.0, 0},
+     {0,   2/6.0, 4/6.0, 0},
+     {0,   1/6.0, 4/6.0, 1/6.0}};
+
+static real matrix_catmull_to_bezier[4][4] =
+    {{0,      1,   4,     1},
+     {-1/6.0, 1, 25/26.0, 1},
+     {0,      1,   5,   5/6.0},
+     {0,      0,   5,   5/6.0}};
+
+static Object *
+fig_read_spline(FILE *file, DiagramData *dia) {
+    int sub_type;
+    int line_style;
+    int thickness;
+    int pen_color;
+    int fill_color;
+    int depth;
+    int pen_style;
+    int area_fill;
+    real style_val;
+    int cap_style;
+    int forward_arrow;
+    int backward_arrow;
+    int npoints;
+    Point *points;
+    GPtrArray *props = g_ptr_array_new();
+    Object *newobj = NULL;
+    BezPoint *bezpoints;
+    int i;
+
+    if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d\n",
+	       &sub_type,
+	       &line_style,
+	       &thickness,
+	       &pen_color,
+	       &fill_color,
+	       &depth,
+	       &pen_style,
+	       &area_fill,
+	       &style_val,
+	       &cap_style,
+	       &forward_arrow,
+	       &backward_arrow,
+	       &npoints) != 13) {
+	message_error(_("Couldn't read spline info: %s\n"), strerror(errno));
+	return NULL;
+    }
+
+    if (forward_arrow == 1) {
+	fig_read_arrow(file);
+    }
+
+    if (backward_arrow == 1) {
+	fig_read_arrow(file);
+    }
+
+    if (!fig_read_n_points(file, npoints, &points)) {
+	return NULL;
+    }
+     
+    switch (sub_type) {
+    case 0: /* Open approximated spline */
+    case 1: /* Closed approximated spline */
+	message_warning(_("Cannot convert approximated spline yet."));
+	goto exit;
+    case 2: /* Open interpolated spline */
+    case 3: /* Closed interpolated spline */
+	/* Despite what the Fig description says, interpolated splines
+	   now also have the line with spline info from the X-spline */
+    case 4: /* Open X-spline */
+    case 5: /* Closed X-spline */
+	{
+	    double f;
+	    gboolean interpolated = TRUE;
+	    for (i = 0; i < npoints; i++) {
+		if (fscanf(file, " %lf ", &f) != 1) {
+		    message_error(_("Couldn't read spline info: %s\n"),
+				  strerror(errno));
+		    goto exit;
+		}
+		if (f != -1.0 && f != 0.0) {
+		    message_warning(_("Cannot convert approximated spline yet."));
+		    interpolated = FALSE;
+		}
+	    }
+	    if (!interpolated)
+		goto exit;
+	    /* Matrix-based conversion not ready yet. */
+#if 0
+	    if (sub_type%2 == 0) {
+		bezpoints = fig_transform_spline(npoints, points, FALSE, f);
+		newobj = create_standard_bezierline(npoints, bezpoints, dia);
+	    } else {
+		points = g_renew(Point, points, npoints+1);
+		points[npoints] = points[0];
+		npoints++;
+		bezpoints = fig_transform_spline(npoints, points, TRUE, f);
+		newobj = create_standard_beziergon(npoints, bezpoints, dia);
+	    }
+#else
+	    if (sub_type%2 == 0) {
+		bezpoints = transform_spline(npoints, points, FALSE);
+		newobj = create_standard_bezierline(npoints, bezpoints, dia);
+	    } else {
+		points = g_renew(Point, points, npoints+1);
+		points[npoints] = points[0];
+		npoints++;
+		bezpoints = transform_spline(npoints, points, TRUE);
+		newobj = create_standard_beziergon(npoints, bezpoints, dia);
+	    }
+#endif
+	}
+	if (newobj == NULL) goto exit;
+	break;
+    default: 
+	message_error(_("Unknown spline subtype: %d\n"), sub_type);
+	goto exit;
+    }
+
+    fig_simple_properties(newobj, line_style, thickness,
+			  pen_color, fill_color, area_fill);
+    /* Pen style field (not used) */
+    /* Style_val (size of dots and dashes) in 1/80 inch*/
+    /* Cap style */
+     
+    /* Depth field */
+    if (compound_stack == NULL)
+	depths[depth] = g_list_prepend(depths[depth], newobj);
+    else
+	if (compound_depth > depth) compound_depth = depth;
+ exit:
+    prop_list_free(props);
+    g_free(points);
+    return newobj;
 }
 
 static Object *
@@ -724,6 +1068,7 @@ fig_read_arc(FILE *file, DiagramData *dia) {
 	newobj = create_standard_arc(x1/FIG_UNIT, y1/FIG_UNIT,
 				     x3/FIG_UNIT, y3/FIG_UNIT,
 				     radius, dia);
+	if (newobj == NULL) return NULL;
 	break;
     default: 
 	message_error(_("Unknown polyline subtype: %d\n"), sub_type);
@@ -747,10 +1092,19 @@ fig_read_arc(FILE *file, DiagramData *dia) {
     return newobj;
 }
 
+static PropDescription xfig_text_descs[] = {
+   { "text", PROP_TYPE_TEXT },
+    PROP_DESC_END
+    /* Can't do the angle */
+    /* Height and length are ignored */
+    /* Flags */
+};
+
 static Object *
 fig_read_text(FILE *file, DiagramData *dia) {
-    Property props[5];
-    int num_props = 0;
+    GPtrArray *props = NULL;
+    TextProperty *tprop;
+
     Object *newobj = NULL;
     int sub_type;
     int color;
@@ -763,7 +1117,7 @@ fig_read_text(FILE *file, DiagramData *dia) {
     real height;
     real length;
     int x, y;
-    char *text_buf;
+    char *text_buf = NULL;
 
     if (fscanf(file, " %d %d %d %d %d %lf %lf %d %lf %lf %d %d",
 	       &sub_type,
@@ -784,37 +1138,31 @@ fig_read_text(FILE *file, DiagramData *dia) {
     /* Skip one space exactly */
     text_buf = fig_read_text_line(file);
 
-    newobj = create_standard_text(x/FIG_UNIT, y/FIG_UNIT, text_buf, dia);
-    g_free(text_buf);
+    newobj = create_standard_text(x, y, dia);
+    if (newobj == NULL) goto exit;
 
-    props[num_props].name = "text_colour";
-    props[num_props].type = PROP_TYPE_COLOUR;
-    PROP_VALUE_COLOUR(props[num_props]) = fig_color(color);
-    num_props++;
-    props[num_props].name = "text_alignment";
-    props[num_props].type = PROP_TYPE_ENUM;
-    PROP_VALUE_ENUM(props[num_props]) = sub_type;
-    num_props++;
-    props[num_props].name = "text_height";
-    props[num_props].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[num_props]) = font_size*2.54/72.0;
-    num_props++;
-    /* Can't do the angle */
-    /* Height and length are ignored */
-    /* Flags */
-    props[num_props].name = "text_font";
-    props[num_props].type = PROP_TYPE_FONT;
-    PROP_VALUE_FONT(props[num_props]) = font_getfont(fig_fonts[font]);
-    num_props++;
-
-    newobj->ops->set_props(newobj, props, num_props);
+    props = prop_list_from_descs(xfig_text_descs,pdtpp_true);
 
+    tprop = g_ptr_array_index(props,0);
+    tprop->text_data = g_strdup(text_buf);
+    /*g_free(text_buf); */
+    tprop->attr.alignment = sub_type;
+    tprop->attr.position.x = x/FIG_UNIT;
+    tprop->attr.position.y = y/FIG_UNIT;
+    tprop->attr.font = dia_font_new_from_legacy_name(fig_fonts[font]);
+    tprop->attr.height = font_size*3.54/72.0;
+    tprop->attr.color = fig_color(color);
+    newobj->ops->set_props(newobj, props);
+    
     /* Depth field */
     if (compound_stack == NULL)
 	depths[depth] = g_list_prepend(depths[depth], newobj);
     else
 	if (compound_depth > depth) compound_depth = depth;
 
+ exit:
+    if (text_buf != NULL) free(text_buf);
+    if (props != NULL) prop_list_free(props);
     return newobj;
 }
 
@@ -855,9 +1203,9 @@ fig_read_object(FILE *file, DiagramData *dia) {
 	return FALSE;
     }
 
-    color.red = (colorvalues & 0x00ff0000)>>16;
-    color.green = (colorvalues & 0x0000ff00)>>8;
-    color.blue = colorvalues & 0x000000ff;
+    color.red = ((colorvalues & 0x00ff0000)>>16) / 255.0;
+    color.green = ((colorvalues & 0x0000ff00)>>8) / 255.0;
+    color.blue = (colorvalues & 0x000000ff) / 255.0;
 
     fig_colors[colornumber-32] = color;
     break;
@@ -876,8 +1224,11 @@ fig_read_object(FILE *file, DiagramData *dia) {
       }
       break;
   case 3: /* Spline which includes closed/open control/interpolated spline. */
-      fig_warn(WARNING_NO_SPLINES);
-      return FALSE;
+      item = fig_read_spline(file, dia);
+      if (item == NULL) {
+	  return FALSE;
+      }
+      break;
   case 4: /* Text. */
       item = fig_read_text(file, dia);
       if (item == NULL) {
@@ -943,7 +1294,7 @@ fig_read_paper_size(FILE *file, DiagramData *dia) {
 
   buf[strlen(buf)-1] = 0; /* Remove trailing newline */
   if ((paper = find_paper(buf)) != -1) {
-    get_paper_info(&dia->paper, paper);
+    get_paper_info(&dia->paper, paper, NULL);
     return TRUE;
   }
 
